package org.bfkj;

import com.ibm.disthub2.impl.client.BaseConfig;
import org.bfkj.application.DataProcess;
import org.bfkj.application.ServiceInputControl;
import org.bfkj.config.*;
import org.bfkj.utils.LogUtils;
import org.bfkj.utils.MapTools;
import org.bfkj.utils.MyDbHelper;
import org.bfkj.utils.ScheduleUtil;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.core.env.Environment;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.io.File;
import java.util.*;

/**
 * 初始化
 */

@SpringBootApplication(exclude = {DataSourceAutoConfiguration.class})
@EnableScheduling
public class MainFactoryApplication implements InitializingBean {


    @Value("${baseConfig.sourceData:true}")
    private String sourceData;

    private final String INSERT_SQL = "INSERT INTO log_success ( success, location, logContent, createtime, serviceId, workId, event, iNDataContent, outDataContent, calculationLocation, dataObjectId) VALUES (?,?,?,?,?,?,?,?,?,?,?)";
    private final String ERROR_SQL = "INSERT INTO log_error ( success, location, logContent, createtime, serviceId, workId, event, iNDataContent, outDataContent, calculationLocation, dataObjectId) VALUES (?,?,?,?,?,?,?,?,?,?,?)";

    private RemoteDB remoteDB;
    private RemoteLogDB remoteLogDB;


    public MainFactoryApplication(RemoteDB remoteDB, RemoteLogDB remoteLogDB) {
        this.remoteDB = remoteDB;
        this.remoteLogDB = remoteLogDB;
    }

    /*服务初始化： 启动服务*/
    public static void main(String[] args) {
        String path = System.getProperty("user.dir"); //获取当前项目的路径
        if (!checkFileDir(path)) {
            System.out.println("配置文件夹检测失败");
            return;
        }
        /*获取初始地址*/
        Map<String, String> systemDefaultParams = getSystemDefaultParams(path);
        ConfigurableApplicationContext context = SpringApplication.run(MainFactoryApplication.class, args);
        Environment env = context.getBean(Environment.class);
        String systemParams = env.getProperty("systemParams");
        if (Objects.nonNull(systemParams) && !"".equals(systemParams)) {
            Map<String, Object> inSystemParams = (Map<String, Object>) MapTools.strToObj(systemParams);
            for (String key : inSystemParams.keySet()) {
                String keyValue = inSystemParams.get(key).toString().replace("path", path);
                if (Objects.isNull(MapTools.strToObj(keyValue))) {
                    systemDefaultParams.put(key, keyValue);
                } else {
                    systemDefaultParams.put(key, keyValue);
                }
            }
            /*设置远程连接、服务地址*/
            AppConfig.serviceURL = systemDefaultParams.get("serviceURL");
            AppConfig.REMOTE_DB_CONNECT = systemDefaultParams.get("remote_db_connect");
        }
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            System.out.println("初始化获取MyDbHelper对象异常");
            return;
        }
        if (!initDB(myDbHelper)) {
            System.out.println("初始化远程数据库失败");
            return;
        }
        Runtime.getRuntime().addShutdownHook(new Thread(MainFactoryApplication::currentWorkExit));
        try {
            Map<String, String> hasDeployNodeIDMap = myDbHelper.queryForObject("select Max(deployNodeID) from deploynode where serviceURL = ? ", AppConfig.serviceURL);
            if (hasDeployNodeIDMap.get("code").equals("-1")) {
                System.out.println("初始化获取机器表信息： 执行SQL失败 " + hasDeployNodeIDMap.get("message"));
                return;
            }
            String workID = hasDeployNodeIDMap.get("returnData");
            if (!"".equals(workID) && !"0".equals(workID)) {
                AppConfig.WORK_ID = workID;
            } else {
                Map<String, String> returnKeyValues = myDbHelper.insertReturnKeyValues("insert into deploynode(serviceURL)  values(?)", AppConfig.serviceURL);
                if (returnKeyValues.get("code").equals("-1")) {
                    System.out.println("根据serviceURL新增入库失败： " + ("insert into deploynode(serviceURL)  values(?)") + " 参数： " + AppConfig.serviceURL);
                    return;
                }
                AppConfig.WORK_ID = returnKeyValues.get("returnData");
                if (!MapTools.isNumber(AppConfig.WORK_ID)) {
                    System.out.println("初始化workID异常： " + AppConfig.WORK_ID);
                    return;
                }
            }
        } catch (Exception e) {
            System.out.println("初始化workID异常： " + LogUtils.getException(e));
            return;
        }

        Map<String, Object> serviceListResult = myDbHelper.queryByParamsReturnList("SELECT serviceID,serviceType FROM serviceinfo WHERE workId = ?", AppConfig.WORK_ID);
        List<Map<String, Object>> serviceList = MapTools.getMapList(serviceListResult);
        if (!serviceListResult.get("code").equals("0") || null == serviceList) {
            System.out.println("初始化获取服务异常： " + serviceListResult.get("message"));
            return;
        }
        int autoCount = 0;
        for (Map<String, Object> map : serviceList) {
            // 开启一个线程，启动主动采集
            Object threadsObj = map.get("serviceType");
            if (Objects.nonNull(threadsObj) && threadsObj.toString().equals("4")) {
                autoCount++;
                Map<String, Object> requestParams = new HashMap<>();
                requestParams.put("serviceId", map.get("serviceID").toString());
                ServiceInputControl.startOrStop(requestParams, "1");
            }
        }
        System.out.println("本机共部署服务个数：" + serviceList.size() + ",启动主动采集个数：" + autoCount);
    }
    /**
     * 获取系统默认参数
     *
     * @return
     */
    private static Map<String, String> getSystemDefaultParams(String path) {
        Map<String, String> paramsMaps = new HashMap<>();
        paramsMaps.put("local_db_connect", "{\"username\":\"bf_dev_epi\",\"password\":\"Bfepi2021(\", \"url\":\"jdbc:sqlite:\" " + path + File.separator + "\"businessCacheLocal.sqlite\", \"driver-class-name\":\"org.sqlite.JDBC\"}");
        paramsMaps.put("work_id", "1");
        paramsMaps.put("serviceURL", "127.0.0.1:8080");
        paramsMaps.put("remote_db_connect", "{\"username\":\"bf_dev_epi\",\"password\":\"Bfepi2021(\",\"url\":\"jdbc:mysql://rm-bp12c6mmk845m73p48o.mysql.rds.aliyuncs.com:3306/test_cen\",\"driver-class-name\":\"com.mysql.cj.jdbc.Driver\"}");
        String logPath = "jdbc:sqlite:log" + File.separator + File.separator + "log.sqlite";
        paramsMaps.put("local_log_connect", "{\"username\":\"bf_dev_epi\",\"password\":\"Bfepi2021(\",\"url\":\"" + logPath + "\", \"driver-class-name\":\"org.sqlite.JDBC\"}");
        return paramsMaps;
    }

    /*系统退出时 需要做的操作*/
    private static void currentWorkExit() {
        ServiceInputControl.ScheduleTaskMaps.values().forEach(ScheduleUtil::cancel);
        //关闭数据库连接
        ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT).close();
        ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_LOG_CONNECT).close();
        System.out.println("系统正常退出");
    }

    /*创建文件夹*/
    private static boolean checkFileDir(String path) {
        //初始化默认文件创建
        List<String> pathList = new ArrayList<>() {{
            add("cache");
            add("plugins");
            add("ftp");
            add("upload");
            add("log");
        }};
        for (String paths : pathList) {
            /*创建文件夹*/
            File pluginPath = new File(path + File.separator + paths);
            if (!pluginPath.exists() && !pluginPath.mkdir()) {
                //todo 系统目录或配置文件检测失败
                return false;
            }
        }
        return true;
    }
    public static boolean initDB(MyDbHelper myDbHelper) {

        Map<String, String> hasAppMap = myDbHelper.queryForObject("select count(1) hasCount  from t_application where app_id =?", "1");
        if (hasAppMap.get("code").equals("-1")) {
            System.out.println("初始化数据库 查询应用信息，查询失败： " + hasAppMap.get("message"));
            return false;
        }
        String hasApp = hasAppMap.get("returnData");
        if (MapTools.isBlank(hasApp) || hasApp.equals("0")) {
            myDbHelper.execute("ALTER TABLE `t_application` AUTO_INCREMENT=1;");
            myDbHelper.execute("INSERT INTO `t_application` (`app_name`, `app_show_id`, `app_show_secret`, `app_code_rule`) VALUES ('底座', 'q7kdjmmaf0kerwpf', '9inu7zpllz1folzsljm498dcpi0lsog1', 'N2');");
        }

        Map<String, String> hasUserApp = myDbHelper.queryForObject("select count(1) hasUser  from t_user where user_id =?", "1");
        if (hasUserApp.get("code").equals("-1")) {
            System.out.println("初始化数据库 查询用户信息信息，查询失败： " + hasAppMap.get("message"));
            return false;
        }
        String hasUser = hasUserApp.get("returnData");
        if (MapTools.isBlank(hasUser) || hasUser.equals("0")) {
            myDbHelper.execute("ALTER TABLE `t_user` AUTO_INCREMENT=1;");
            myDbHelper.execute("INSERT INTO `t_user` (`user_name`, `user_pwd`) VALUES ( 'zhaoke', 'cab264992e7cf0de822013e569a0fec5');");
        }

        Map<String, String> hasAuthApp = myDbHelper.queryForObject("select count(1) hasAuth  from t_auth");
        if (hasAuthApp.get("code").equals("-1")) {
            System.out.println("初始化数据库 查询权限信息信息，查询失败： " + hasAppMap.get("message"));
            return false;
        }
        String hasAuth = hasAuthApp.get("returnData");
        if (MapTools.isBlank(hasAuth) || hasAuth.equals("0")) {
            myDbHelper.execute("ALTER TABLE `t_auth` AUTO_INCREMENT=1;");
            myDbHelper.execute("INSERT INTO `t_auth` (`app_id`, `auth_name`, `auth_type`, `up_auth_id`, `auth_ident`, `auth_status`, `auth_comment`, `show_type`, `show_icon`, `show_index`, `is_link`, `route_info`, `file_link`, `queryTemplateID`, `is_show`, `code`, `queryparams`) VALUES (1, '系统设置', 1, NULL, 'systemSetting', NULL, NULL, 1, NULL, NULL, NULL, '/systemSetting', NULL, NULL, 1, NULL, NULL);");
        }

        Map<String, String> hasServiceApp = myDbHelper.queryForObject("select count(1) hasService  from serviceinfo");
        if (hasServiceApp.get("code").equals("-1")) {
            System.out.println("初始化数据库 查询用户信息信息，查询失败： " + hasAppMap.get("message"));
            return false;
        }

        //初始化服务
        String hasService = hasServiceApp.get("returnData");
        if (MapTools.isBlank(hasService) || hasService.equals("0")) {
            myDbHelper.execute("ALTER TABLE `serviceinfo` AUTO_INCREMENT=1;");
            myDbHelper.execute("INSERT INTO `serviceinfo` (`serviceName`, `serviceDescribe`, `serviceType`) VALUES ( '获取全部权限', '获取全部权限', 1);");
        }
        try {
            //远程表日志表结构结构
            myDbHelper.execute("CREATE TABLE log_error (id int NOT NULL auto_increment  PRIMARY KEY, success varchar(8) DEFAULT NULL, location varchar(256) DEFAULT NULL,  logContent text,  serviceId int DEFAULT NULL,  workId int DEFAULT NULL,  event varchar(2) DEFAULT NULL,  iNDataContent longtext,  outDataContent text,  calculationLocation varchar(256) DEFAULT NULL,  dataObjectId varchar(32) DEFAULT NULL,  createtime datetime DEFAULT NULL");
            myDbHelper.execute("CREATE TABLE log_success (id int NOT NULL auto_increment  PRIMARY KEY, success varchar(8) DEFAULT NULL, location varchar(256) DEFAULT NULL,  logContent text,  serviceId int DEFAULT NULL,  workId int DEFAULT NULL,  event varchar(2) DEFAULT NULL,  iNDataContent longtext,  outDataContent text,  calculationLocation varchar(256) DEFAULT NULL,  dataObjectId varchar(32) DEFAULT NULL,  createtime datetime DEFAULT NULL");
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    @Scheduled(cron = "0 15 * * * ?")
    public void closeObject() {
        long timeOut = 15 * 1000 * 60;
        Collection<DataProcess> tpList = ObjectMap.dataProcessMaps.values();
        for (DataProcess dataProcess : tpList ) {
            if (System.currentTimeMillis() - dataProcess.getLastActive() > timeOut) {
                String serviceId = dataProcess.getServiceId();
                ObjectMap.getordropInput(serviceId, false);
            }
        }
    }


    /*1每一分钟手动释放资源*/
    @Scheduled(cron = "0 1 * * * ?")
    public void clearGc() {
        Runtime.getRuntime().gc();
    }

    /*1每2秒执行一次 日志的批量插入*/
    @Scheduled(cron = "*/2 * * * * *")
    public void batchLog() {
        Map<String, List<Object>> logSuccessMap = LogUtils.logSuccessMapList;
        Map<String, List<Object>> logErrorMap = LogUtils.logErrorMapList;
        /*批量写日志*/
        if (logSuccessMap.size() == 0 && logErrorMap.size() == 0) {
            return;
        }
        long current = System.currentTimeMillis();
        List<String> successList = logSuccessMap.keySet().stream().filter(key -> Long.parseLong(key) < current).toList();
        List<Object> logSuccessList = new ArrayList<>();
        for (String currentTime : successList) {
            logSuccessList.addAll(logSuccessMap.get(currentTime));
            logSuccessMap.remove(currentTime);
        }
        List<String> errorList = logErrorMap.keySet().stream().filter(key -> Long.parseLong(key) < current).toList();
        List<Object> logErrorList = new ArrayList<>();
        for (String currentTime : errorList) {
            logErrorList.addAll(logErrorMap.get(currentTime));
            logErrorMap.remove(currentTime);
        }
        MyDbHelper logDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_LOG_CONNECT);
        if (Objects.nonNull(logDbHelper.getErrorMessage())) {
            System.out.println("获取日志数据库失败");
            return;
        }

        try {
            if (logSuccessList.size() > 0) {
                logDbHelper.JDBCBatch(INSERT_SQL, logSuccessList, null);
            }
            if (logErrorList.size() > 0) {
                logDbHelper.JDBCBatch(ERROR_SQL, logErrorList, null);
            }
        } catch (Exception e) {
            System.out.println("日志记录异常" + LogUtils.getException(e));
        }
    }

    @Override
    @SuppressWarnings("unchecked")
    public void afterPropertiesSet() {
        try {
            Map<String, Object> mapDB = (Map<String, Object>) MapTools.strToObj(MapTools.objToJSONStr(remoteDB));
            mapDB.put("driver-class-name", mapDB.get("driverClassName"));
            AppConfig.REMOTE_DB_CONNECT = MapTools.objToJSONStr(mapDB);

            Map<String, Object> maplogDB = (Map<String, Object>) MapTools.strToObj(MapTools.objToJSONStr(remoteLogDB));
            maplogDB.put("driver-class-name", maplogDB.get("driverClassName"));
            AppConfig.REMOTE_DB_LOG_CONNECT = MapTools.objToJSONStr(maplogDB);
            AppConfig.serviceURL = remoteDB.getServiceURL();
            AppConfig.SOURCE_DATA = sourceData;

        } catch (Exception e) {
            System.out.println("配置文件中，连接数据库信息错误");
        }
    }
}