package org.bfkj.api;

import org.bfkj.application.AuthApplicationImpl;
import org.bfkj.application.DataProcess;
import org.bfkj.config.AppConfig;
import org.bfkj.config.ObjectMap;
import org.bfkj.utils.ImportExcelUtils;
import org.bfkj.utils.MapTools;
import org.bfkj.utils.MyDbHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 统一入口
 */
@RestController
@RequestMapping("openApi")
//@CrossOrigin
public class CommonApi {

    @Autowired
    private AuthApplicationImpl authApplication;
    @Autowired
    private ImportExcelUtils importExcelUtils;

    /**
     * 服务数据恢复 ： 因为日志的数据内容拆分为4个字段，所有此方法需要调整可
     */
    @PostMapping("abnormalDataRecovery")
    public Map<String, Object> abnormalDataRecovery(@RequestHeader Map<String, String> headers, @RequestBody Map<String, Object> params) {
        String verificationToke = authApplication.verificationUserToken(headers);
        if (!MapTools.isBlank(verificationToke)) {
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "用户令牌验证失败：" + verificationToke);
            return errMap;
        }
        return authApplication.abnormalDataRecovery(params);
    }

    /**
     * 查询入口CRUD
     */
    @PostMapping("query")
    @ResponseBody
    public Object queryApi(@RequestHeader Map<String, String> headers, @RequestBody Map<String, Object> params) {
        Map<String, Object> stringObjectMap;
        if (params.containsKey("id") && !params.containsKey("serviceId")) {
            Map<String, Object> callBack = new HashMap<>();
            callBack.put("serviceId", "200241");
            callBack.put("dataContent", params);
            callBack.put("event", "0");
            stringObjectMap = commonInterface(headers, callBack, "0");
        } else {
            stringObjectMap = commonInterface(headers, params, "0");
        }
        return stringObjectMap;
    }

    /**
     * 新增入口CRUD
     */
    @PostMapping("newdata")
    @ResponseBody
    public Object newDataApi(@RequestHeader Map<String, String> headers, @RequestBody Map<String, Object> params) {
        return commonInterface(headers, params, "1");
    }

    /**
     * /**
     * 更新入口CRUD
     */
    @PostMapping("modifydata")
    @ResponseBody
    public Object modifyDataApi(@RequestHeader Map<String, String> token, @RequestBody Map<String, Object> params) {
        return commonInterface(token, params, "2");
    }

    /**
     * /**
     * /**
     * 删除入口CRUD
     */
    @PostMapping("movedata")
    @ResponseBody
    public Object moveDataApi(@RequestHeader Map<String, String> headers, @RequestBody Map<String, Object> params) {
        return commonInterface(headers, params, "3");
    }

    /**
     * 统一接收处理
     */
    private Map<String, Object> commonInterface(Map<String, String> headers, Map<String, Object> params, String defaultEvent) {
        if (!params.containsKey("id")) {
            if (headers.containsKey("token")) {
                String appToken = headers.get("token");
                String verificationToke = authApplication.verificationToken(appToken);
                if (!MapTools.isNumber(verificationToke)) {
                    HashMap<String, Object> errMap = new HashMap<>();
                    errMap.put("code", "-1");
                    errMap.put("message", "应用token验证失败：" + verificationToke);
                    return errMap;
                }
                params.remove("id");
            } else {
                String verificationToke = authApplication.verificationUserToken(headers);
                if (!MapTools.isBlank(verificationToke)) {
                    HashMap<String, Object> errMap = new HashMap<>();
                    errMap.put("code", "-1");
                    errMap.put("message", "用户令牌验证失败：" + verificationToke);
                    return errMap;
                }
            }
        }
        Object event = params.get("event");//获取参数中的事件标识
        if (Objects.isNull(event) && !MapTools.isBlank(defaultEvent)) {//如果参数中的没有事件标识并且有默认事件标识则使用默认事件标识
            event = defaultEvent;
            params.put("event", defaultEvent);
        }
        if (!MapTools.isBlank(defaultEvent) && !event.equals(defaultEvent)) {//如果有默认事件标识且传入的事件标识与默认事件标识不相同则返回错误
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "当前接口只支持event为" + defaultEvent + "，当前接收参数为：" + params);
            return errMap;

        }
        Object serviceId = params.get("serviceId");
        if (Objects.isNull(serviceId) || !params.containsKey("event")) {
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "缺失必要的参数：服务编号serviceId或事件标识event" + params);
            return errMap;
        }
        //通过服务ID 查询对应服务的服务类型是否为1，如果时则继续，否则返回
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "服务：" + serviceId + " 获取数据库对象失败，异常信息 " + myDbHelper.getErrorMessage());
            return errMap;
        }
        Map<String, String> hasServiceMap = myDbHelper.queryForObject("select count(1) count from serviceinfo  where serviceID=? and serviceType = 1", serviceId);
        if (hasServiceMap.get("code").equals("-1")) {
            System.out.println("数据接受获取服务信息，查询失败： " + hasServiceMap.get("message"));
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "数据接受获取服务信息，查询失败： " + hasServiceMap.get("message"));
            return errMap;
        }
        String count = hasServiceMap.get("returnData");
        if ("".equals(count) || "0".equals(count)) {
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "服务：" + serviceId + "是采集服务不支持操作");
            return errMap;
        }
        DataProcess dataProcessObj = ObjectMap.getordropInput(serviceId.toString(), true);//获取数据输出处理对象
        if (null == dataProcessObj || !MapTools.isBlank(dataProcessObj.getErrorMessage())) {
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "服务不可用：" + (null == dataProcessObj ? "" : dataProcessObj.getErrorMessage()));
            return errMap;
        }
        return dataProcessObj.processData(params, headers.get("user_id"));
    }


    @PostMapping("/upload")
    public Object upload(@RequestParam(value = "file") MultipartFile excelFile, @RequestParam(value = "serviceId") String serviceID ,@RequestHeader Map<String,Object> headers) throws Exception {
        if (excelFile.isEmpty()) {
            return MapTools.processFail("文件夹为空，重新上传");
        } else {
            String userName = "";
            if (headers.containsKey("user_id")){
                MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
                Map<String, Object> userId = myDbHelper.queryByParamsReturnList("SELECT user_name FROM t_user  WHERE user_id = ?", Objects.toString(headers.get("user_id")));
                Object o = userId.get("returnData");
                if (o instanceof List<?> tpList  && !tpList.isEmpty() && tpList.get(0) instanceof Map currentMap ){
                    userName = Objects.toString(currentMap.get("user_name"));
                }

            }

            String fileName = excelFile.getOriginalFilename();
            InputStream is = excelFile.getInputStream();
            Map<String, Object> uploadFile = importExcelUtils.upload(fileName, is,userName);
            if ("-1".equals(uploadFile.get("code"))) {
                return uploadFile;
            }
            Map<String, Object> sendData = new HashMap<>();
            sendData.put("serviceId", serviceID);
            sendData.put("dataContent", uploadFile.get("returnData"));
            DataProcess dataProcessObj = ObjectMap.getordropInput(serviceID, true);//获取数据输出处理对象
            if (null == dataProcessObj || !MapTools.isBlank(dataProcessObj.getErrorMessage())) {
                HashMap<String, Object> errMap = new HashMap<>();
                errMap.put("code", "-1");
                errMap.put("message", "服务不可用：" + (null == dataProcessObj ? "" : dataProcessObj.getErrorMessage()));
                return errMap;
            }
            dataProcessObj.processData(sendData);
            return MapTools.processSuccess("上传成功");
        }
    }
}
