package org.bfkj.api;

import org.bfkj.application.AuthApplicationImpl;
import org.bfkj.application.DataProcess;
import org.bfkj.config.AppConfig;
import org.bfkj.config.ObjectMap;
import org.bfkj.utils.MapTools;
import org.bfkj.utils.MyDbHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 统一入口
 */
@RestController
@RequestMapping("newOpenApi")
//@CrossOrigin
public class NewCommonApi {
    @Autowired
    private AuthApplicationImpl authApplication;

    @PostMapping("search")
    @ResponseBody
    public Object search(@RequestHeader Map<String, String> headers, @RequestBody Map<String, Object> params) {
        return commonInterface(headers, params, "0", (params.containsKey("id") && !params.containsKey("serviceId")) ? Objects.toString(params.get("id")) : null);
    }
    /**
     * 统一接收处理
     */
    private Map<String, Object> commonInterface(Map<String, String> headers, Map<String, Object> params, String defaultEvent, String... queryId) {
        if (Objects.isNull(queryId) || Objects.isNull(queryId[0])) {
            if (headers.containsKey("token")) {
                String appToken = headers.get("token");
                String verificationToke = authApplication.verificationToken(appToken);
                if (!MapTools.isNumber(verificationToke)) {
                    HashMap<String, Object> errMap = new HashMap<>();
                    errMap.put("code", "-1");
                    errMap.put("message", "应用token验证失败：" + verificationToke);
                    return errMap;
                }
            } else {
                String verificationToke = authApplication.verificationUserToken(headers);
                if (!MapTools.isBlank(verificationToke)) {
                    HashMap<String, Object> errMap = new HashMap<>();
                    errMap.put("code", "-1");
                    errMap.put("message", "用户令牌验证失败：" + verificationToke);
                    return errMap;
                }
            }
        }
        Object event = params.get("event");//获取参数中的事件标识
        if (Objects.isNull(event) && !MapTools.isBlank(defaultEvent)) {//如果参数中的没有事件标识并且有默认事件标识则使用默认事件标识
            event = defaultEvent;
            params.put("event", defaultEvent);
        }
        if (!MapTools.isBlank(defaultEvent) && !event.equals(defaultEvent)) {//如果有默认事件标识且传入的事件标识与默认事件标识不相同则返回错误
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "当前接口只支持event为" + defaultEvent + "，当前接收参数为：" + params);
            return errMap;

        }
        Object serviceId = Objects.nonNull(queryId) && Objects.nonNull(queryId[0]) ? (Integer.parseInt(queryId[0]) + 10000) : params.get("serviceId");
        if (Objects.isNull(serviceId) || !params.containsKey("event")) {
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "缺失必要的参数：服务编号serviceId或事件标识event" + params);
            return errMap;
        }
        //通过服务ID 查询对应服务的服务类型是否为1，如果时则继续，否则返回
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "服务：" + serviceId + " 获取数据库对象失败，异常信息 " + myDbHelper.getErrorMessage());
            return errMap;
        }
        Map<String, String> hasServiceMap = myDbHelper.queryForObject("select count(1) count from serviceinfo  where serviceID=? and serviceType = 1", serviceId);
        if (hasServiceMap.get("code").equals("-1")) {
            System.out.println("数据接受获取服务信息，查询失败： " + hasServiceMap.get("message"));
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "数据接受获取服务信息，查询失败： " + hasServiceMap.get("message"));
            return errMap;
        }
        String count = hasServiceMap.get("returnData");
        if ("".equals(count) || "0".equals(count)) {
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "服务：" + serviceId + "是采集服务不支持操作");
            return errMap;
        }
        DataProcess dataProcessObj = ObjectMap.getordropInput(serviceId.toString(), true);//获取数据输出处理对象
        if (null == dataProcessObj || !MapTools.isBlank(dataProcessObj.getErrorMessage())) {
            HashMap<String, Object> errMap = new HashMap<>();
            errMap.put("code", "-1");
            errMap.put("message", "服务不可用：" + (null == dataProcessObj ? "" : dataProcessObj.getErrorMessage()));
            return errMap;
        }
        return dataProcessObj.processData(params, headers.get("user_id"));
    }
}
