package org.bfkj.application;


import com.fasterxml.jackson.databind.ObjectMapper;
import org.bfkj.config.AppConfig;
import org.bfkj.config.ObjectMap;
import org.bfkj.utils.LogUtils;
import org.bfkj.utils.MapTools;
import org.bfkj.utils.MyDbHelper;
import org.bfkj.utils.RandomGraphic;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class AuthApplicationImpl {

    //统一成功信息处理
    public Map<String, Object> processSuccess(Object returnData) {
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "0");
        returnMap.put("returnData", returnData);
        return returnMap;
    }

    //统一错误信息处理
    public Map<String, Object> processFail(String errorMessage) {
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "-1");
        returnMap.put("message", errorMessage);
        return returnMap;
    }


    /**
     * 获取应用令牌
     *
     * @param params
     * @param sessionID
     * @return
     */
    public Object getToken(Map<String, String> params, String sessionID) {
        String appid = params.get("appid");
        String appSecret = params.get("appSecret");
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("获取数据库连接失败");
        }
        Map<String, Object> appInfoResult = myDbHelper.queryByParamsReturnList("select app_code_rule,app_id,app_name,app_log, app_token_effective,active_duration from t_application where  app_show_id =? AND  app_show_secret=?", appid, appSecret);
        if (!appInfoResult.get("code").equals("0")) {
            return processFail("查询异常" + appInfoResult.get("message"));
        }
        List<Map<String, Object>> appData = (List<Map<String, Object>>) appInfoResult.get("returnData");
        if (appData.isEmpty()) {
            return processFail("错误的appid或appSecret");
        }
//        无条件删除已经过期的令牌delete from 应用验证表 where app_effective > NOW()
        myDbHelper.updateByCondition("delete from t_application_login where app_effective > NOW()", null);
        // 生成全局唯一的应用令牌
        String appTokenOnly = createLifeCycleCol(9999L, 9999);
        /*当前请求ID*/
        String serviceOnlyID = MapTools.getMD5Str(sessionID);
        //创建当前请求的令牌
        Map<String, Object> appInfoData = appData.get(0); //apptoken 有效时长，用户有效存活时长
        String app_id = appInfoData.get("app_id").toString();
        Map<String, Object> insertReturnKeyValues = myDbHelper.updateByCondition("insert into t_application_login(app_id,app_token,app_effective,session_id,request_time) values(?,?," + (Objects.nonNull(appInfoData.get("app_token_effective")) ? "" + appInfoData.get("app_token_effective") + " HOUR)" : null) + " ,?,?)", null, app_id, appTokenOnly, serviceOnlyID, MapTools.dateTostr(new Date()));
        if (Objects.isNull(insertReturnKeyValues) || insertReturnKeyValues.get("code").equals("-1")) {
            return processFail("创建应用Token异常");
        }
        Map<String, Object> listResult = myDbHelper.queryByParamsReturnList("select * from t_application_login where  app_token=?  ", appTokenOnly);
        HashMap<String, Object> returnData = new HashMap<>();
        returnData.put("app_token", appTokenOnly);
        returnData.put("app_effective", ((List<Map<String, Object>>) listResult.get("returnData")).get(0).get("app_effective")); //令牌有效期
        returnData.put("active_duration", appInfoData.get("active_duration"));
        returnData.put("app_name", appInfoData.get("app_name"));
        returnData.put("app_log", appInfoData.get("app_log"));
        returnData.put("app_code_rule", appInfoData.get("app_code_rule"));
        return processSuccess(returnData);
    }

    /**
     * 校验应用令牌的逻辑(app_token) 返回字符串----令牌唯一性确保无需其它信息进行判断
     *
     * @param appToken
     * @return
     */
    public String verificationToken(String appToken) {
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return "验证令牌获取远程数据库连接失败" + myDbHelper.getErrorMessage();
        }
        Map<String, Object> verificationTokenResult = myDbHelper.queryByParamsReturnList("select app_id from t_application_login where app_token=? and  ( app_effective > NOW() or  app_effective is NULL )", appToken);
        List<Map<String, Object>> returnData = MapTools.getMapList(verificationTokenResult);
        return (!verificationTokenResult.get("code").equals("0") || Objects.isNull(returnData) || returnData.isEmpty()) ? "查询错误或令牌验证不通过" : returnData.get(0).get("app_id").toString();
    }

    /**
     * 刷新应用令牌的逻辑：
     *
     * @param params
     * @return
     */
    public Object refreshToken(Map<String, String> params) {
        String app_token = params.get("app_token");
        if (Objects.isNull(app_token)) {
            return processFail("app_token为空，刷新失败");

        }
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("刷新应用app_token获取远程数据库连接失败" + myDbHelper.getErrorMessage());
        }
        String appid = verificationToken(app_token);
        if (!MapTools.isNumber(appid)) {
            return processFail("校验token失败: " + appid);
        }
        // 获取令牌有效时长
        Map<String, Object> appTokenEffectiveResult = myDbHelper.queryByParamsReturnList("select app_token_effective from t_application where  app_id = ?", appid);
        List<Map<String, Object>> returnData = MapTools.getMapList(appTokenEffectiveResult);
        if (null == returnData || returnData.isEmpty()) {
            return appTokenEffectiveResult;
        }

        Object app_token_effective = returnData.get(0).get("app_token_effective");
        String app_effective = Objects.nonNull(app_token_effective) ? "DATE_ADD(NOW(),INTERVAL  " + app_token_effective + " HOUR)" : null;
        try {
            myDbHelper.updateByCondition("update t_application_login set app_effective=" + app_effective + "  where app_token=?", null, app_token);
            Map<String, Object> listResult = myDbHelper.queryByParamsReturnList("select * from t_application_login where  app_token=?  ", app_token);
            List<Map<String, Object>> returnD = MapTools.getMapList(listResult);
            if (null == returnD || returnD.isEmpty()) {
                return listResult;
            }
            HashMap<String, Object> returnMap = new HashMap<>();
            returnMap.put("app_token", app_token);
            returnMap.put("app_effective", returnD.get(0).get("app_effective"));
            return processSuccess(returnMap);
        } catch (Exception e) {
            return processFail("app_token 刷新失败");

        }
    }

    /**
     * 获取验证码
     *
     * @param params {appId:"",token:""}
     * @return 验证码执行结果信息
     */
    public Object getVCode(Map<String, Object> params) {
        Object app_token = params.get("app_token");
        if (Objects.isNull(app_token)) {
            return processFail("app_token为空");
        }
        String appid = verificationToken(app_token.toString());
        if (!MapTools.isNumber(appid)) {
            return processFail("tokan校验失败：" + appid);
        }
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("获取验证码，获取数据库连接失败");
        }
        Map<String, Object> appCodeRuleResult = myDbHelper.queryByParamsReturnList("SELECT  app_code_rule ,app_code_effective   FROM t_application WHERE app_id =?", appid);
        List<Map<String, Object>> returnD = MapTools.getMapList(appCodeRuleResult);
        if (appCodeRuleResult.get("code").equals("-1")) {
            return processFail("获取验证规则失败");
        }
        // 组建验证码规则
        Object appRule = null;
        int appCodeEffective = 0;
        if (Objects.nonNull(returnD) && returnD.size() > 0) {
            appRule = returnD.get(0).get("app_code_rule");
            appCodeEffective = Objects.isNull(returnD.get(0).get("app_code_effective")) ? 300 : Integer.parseInt(returnD.get(0).get("app_code_effective").toString());
        }
        if (Objects.isNull(appRule)) {
            return processFail("无需验证码");
        }

        String codeFormat = "";
        //codeType == 1 ? "0123456789" : "0123456789ACDEFGHJKLMNPQRSTWXY"
        if (appRule.toString().contains("A")) {
            codeFormat = codeFormat + "ACDEFGHJKLMNPQRSTWXY";
        } else if (appRule.toString().contains("N")) {
            codeFormat = codeFormat + "0123456789";
        } else {
            return processFail("验证码规则不存在");

        }
        // 生成验证码
        String appRuleStr = appRule.toString();  //A2N4
        Map<String, Object> map = RandomGraphic.generateVerifyCode(Integer.parseInt(appRuleStr.substring(appRuleStr.length() - 1)), codeFormat);
        if (null == map) {
            return processFail("生成验证码失败");
        }
        try {
            myDbHelper.updateByCondition("update t_application_login set app_code = ?,app_code_expire = DATE_ADD(NOW(),INTERVAL  " + appCodeEffective + " SECOND )  where app_token=?", null, map.get("verifyCode").toString(), app_token);
            Map<String, Object> listResult = myDbHelper.queryByParamsReturnList("select * from t_application_login where  app_token=?  ", app_token);
            List<Map<String, Object>> applicationInfo = MapTools.getMapList(listResult);
            if (listResult.get("code").equals("-1") || null == applicationInfo) {
                return processFail("获取验证规则失败");

            }
            return processSuccess(new HashMap<String, Object>() {{
                put("verifyCode", map.get("verifyCodeImage"));
                put("app_code_expire", applicationInfo.get(0).get("app_code_expire"));

            }});
        } catch (Exception e) {
            return processFail("验证码异常：" + LogUtils.getException(e));
        }
    }

    /**
     * 用户登录并获取用户权限信息
     *
     * @param data{”token“：”“，”appid“：”“，”username“：”“，”password“：”“}
     * @return 获取用户权限信息
     */
    public Map<String, Object> signIn(Map<String, String> data) {
        String username = data.get("username");
        String password = data.get("password");
        String verificationCode = data.get("verifyCode");
        String app_token = data.get("app_token");
        String app_id = verificationToken(app_token);
        if (!MapTools.isNumber(app_id)) {
            return processFail("登陆token校验失败：" + app_id);
        }
        //校验验证码
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("用户登陆，获取数据库连接失败");
        }
        Map<String, Object> applicationLogin = myDbHelper.queryByParamsReturnList("select 1 from t_application_login where app_token=? and if(app_code is null,true,app_code = ? and app_code_expire > NOW())", app_token, verificationCode);
        List<Map<String, Object>> applicationList = MapTools.getMapList(applicationLogin);


        if (!applicationLogin.get("code").equals("0")) {
            return processFail("校验验证码，执行错误");
        }
        if (applicationList == null || applicationList.isEmpty()) {
            return processFail("验证码错误或过期");
        }
        Map<String, Object> userInfoResult = myDbHelper.queryByParamsReturnList("select user_id,token_valid_duration from t_user where user_name = ?  and user_pwd =?", username, password);
        if (!userInfoResult.get("code").equals("0")) {
            return processFail("校验用户信息，执行异常");
        }
        List<Map<String, Object>> userInfoList = MapTools.getMapList(userInfoResult);
        if (null == userInfoList || userInfoList.isEmpty()) {
            return processFail("用户名或密码错误");
        }
        String userID = userInfoList.get(0).get("user_id").toString();
        String userToken = MapTools.getMD5Str(new Date().getTime() + "");
        try {
            Object token_valid_duration = userInfoList.get(0).get("token_valid_duration");
            myDbHelper.updateByCondition("update t_user set user_token = ?,user_status = 1,signInTime = NOW(),token_valid_time = if(" + Objects.isNull(token_valid_duration) + ",null, DATE_ADD(NOW(),INTERVAL   " + (Objects.isNull(token_valid_duration) ? 0 : Integer.parseInt(token_valid_duration.toString())) + " SECOND) )  where user_id= ? ", null, userToken, userID);
            return processSuccess(new HashMap<String, Object>() {{
                put("user_token", userToken);
                put("user_id", userID);
            }});
        } catch (Exception e) {
            return processFail("更新用户Token失败");
        }
    }

    public String verificationUserToken(Map<String, String> params) {
        String user_token = params.get("user_token");
        String user_id = params.get("user_id");
        if (Objects.isNull(user_token) || Objects.isNull(user_id)) return "user_token 或则 user_id为空";
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return "验证user_token获取远程数据库连接失败" + myDbHelper.getErrorMessage();
        }
        Map<String, String> userCountMap = myDbHelper.queryForObject("select count(1) as count from t_user where user_id=? and user_token = ? and (token_valid_time > NOW() or  token_valid_time is null )", user_id, user_token);
        if ((!userCountMap.get("code").equals("0") || Objects.isNull(userCountMap.get("returnData")) || "0".equals(userCountMap.get("returnData")))) {
            return "查询错误或令牌验证不通过";
        }
        return null;
    }

    /**
     * 用户登出
     *
     * @param params
     * @return
     */
    public Map<String, Object> loginOut(Map<String, String> params) {
        String errorInfo = verificationUserToken(params);
        if (Objects.nonNull(errorInfo)) {
            return processFail("用户登出失败： " + errorInfo);
        }
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("用户登出,获取远程数据库连接失败" + myDbHelper.getErrorMessage());
        }
        try {
            String user_id = params.get("user_id");
            myDbHelper.updateByCondition("update t_user set user_token = null,user_status = 0,LogoutTime = NOW() where user_id=?", null, user_id);
            return processSuccess("用户登出成功");
        } catch (Exception e) {
            return processFail("用户登出失败： ");
        }
    }

    public Map<String, Object> getAuth(Map<String, String> params) {
        String errorInfo = verificationUserToken(params); //user_toke,user_id
        if (Objects.nonNull(errorInfo)) {
            return processFail("获取权限： " + errorInfo);
        }
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("获取用户权限,获取远程数据库连接失败" + myDbHelper.getErrorMessage());
        }
        String user_id = params.get("user_id");
        if (user_id.equals("1")) { // 如果是内置管理员 则获取全部权限
            return myDbHelper.queryByParamsReturnList(" select * from t_auth");
        }
        return myDbHelper.queryByParamsReturnList(" select * from t_auth");
        // 从user_auth表中获取导航栏权限： select auth_id from t_user_auth  where user_id = ?
//        return myDbHelper.queryByParamsReturnList(" select TA.* from t_user_auth  TUA LEFT JOIN t_auth TA  ON TA.auth_id = TUA.auth_id where TUA.user_id = ?",user_id);
    }

    public Map<String, Object> initColumnSet(Map<String, Object> data) {
        if (!data.containsKey("auth_id")) {
            return processFail("权限编号为空");
        }
        String auth_id = data.get("auth_id").toString();
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("初始化列设置：权限编号:" + auth_id + " 获取数据库对象异常");
        }

        Map<String, Object> serviceInfoResult = myDbHelper.queryByParamsReturnList("select serviceID,serviceType from serviceinfo where serviceID = (select max(serviceID) as serviceID from t_auth where auth_id = ?) and serviceType ='1'", auth_id);
        List<Map<String, Object>> serviceInfoList = MapTools.getMapList(serviceInfoResult);

        if (!"0".equals(serviceInfoResult.get("code"))) {
            return processFail("初始化列设置：权限编号:" + auth_id + " 获取服务列表异常");
        }
        if (Objects.isNull(serviceInfoList) || serviceInfoList.isEmpty()) {
            return processFail("初始化列设置：权限编号:" + auth_id + " 未找到对应的服务或者服务为采集类，不可进行列设置初始化");
        }
        String serviceID = serviceInfoList.get(0).get("serviceID").toString();
        // 通过服务获取列设置
        List<Map<String, Object>> columnList;
        Object tableName = null;
        try {
            Map<String, Object> queryByParamsReturnMap = myDbHelper.queryByParamsReturnList("select calculation_library.*,datasourceinfo.connectConfig  from calculation_library INNER JOIN datasourceinfo on datasourceinfo.dataSourceID = calculation_library.datasource_id  where service_id=? and library_type =3 order by library_sort, library_id limit 1", serviceID);
            if (queryByParamsReturnMap.get("code").equals("-1")) {
                return processFail("通过服务获取列设置：SQL执行异常");
            }
            List<Map<String, Object>> queryByParamsReturnList = MapTools.getMapList(queryByParamsReturnMap);
            if (Objects.isNull(queryByParamsReturnList) || queryByParamsReturnList.isEmpty()) {
                return processFail("当前服务暂时不支持列设置初始化");
            }
            Map<String, Object> libraryInfo = queryByParamsReturnList.get(0);
            tableName = libraryInfo.get("sourceObjectName");
            Object sql = libraryInfo.get("computing_expression");
            if (Objects.isNull(sql) && Objects.isNull(tableName)) {
                return processFail("服务配置错误");
            }
            Map<String, Object> specialSearchDataResult = new HashMap<>();
            String connectConfig = libraryInfo.get("connectConfig").toString();
            MyDbHelper remoteMyDbHelper = ObjectMap.getordropMyDbHelper(connectConfig);
            if (Objects.nonNull(remoteMyDbHelper.getErrorMessage())) {
                return processFail("初始化列设置：权限编号:" + auth_id + " 获取数据库对象异常");
            }
            if (Objects.nonNull(sql)) {
                String tempSQL = sql.toString();
                //todo 获取对应的数据源 获取mydb对象
                // 替换SQL中的符号
                String expression = "(?<=《)([^》]+)?(?=》)";
                Pattern regExpression = Pattern.compile(expression);  // select * from user where userid = 《userID》 select * from user where userid is null
                Matcher parameterNames = regExpression.matcher(tempSQL);
                while (parameterNames.find()) {
                    String tempColumn = parameterNames.group(); //
                    tempSQL = tempSQL.replaceFirst("《" + tempColumn + "》", " null ");
                }
                specialSearchDataResult = remoteMyDbHelper.getAllColumnForSet(tempSQL);
            } else {
                specialSearchDataResult = remoteMyDbHelper.getAllColumnTableSet(tableName.toString());
            }
            if (!"0".equals(specialSearchDataResult.get("code"))) {
                return processFail("初始化列设置：权限编号:" + auth_id + " 模拟查询异常：" + specialSearchDataResult.get("message"));
            }
            columnList = MapTools.getMapList(specialSearchDataResult);
            if (Objects.isNull(columnList) || columnList.isEmpty()) {
                return processFail("当前服务暂时不支持列设置初始化");
            }
        } catch (Exception e) {
            return processFail("获取服务对应算法执行异常" + LogUtils.getException(e));
        }
        Map<String, Object> columnSetListResult = myDbHelper.queryByParamsReturnList("select  * from querytemplatecolumnset where serviceOutPutId=?", auth_id);
        List<Map<String, Object>> columnSetList = MapTools.getMapList(columnSetListResult);
        if (!"0".equals(columnSetListResult.get("code")) || null == columnSetList) {
            return processFail("初始化列设置：权限编号:" + auth_id + " 获取权限列标志异常" + columnSetListResult.get("message"));
        }

        //初始化一个List<string> OLDColumnList--- > user(userPWD)-- 数据库元数据
        List<String> OLDColumnList = new ArrayList<>();
        HashMap<Object, Map<String, Object>> tempColumnSet = new HashMap<>();
        for (Map<String, Object> currentMap : columnSetList) {
            OLDColumnList.add(currentMap.get("columnName").toString());
            tempColumnSet.put(currentMap.get("columnName").toString(), currentMap);
        }
        List<Map<String, Object>> insertList = new ArrayList<>();
        for (Map<String, Object> columnMap : columnList) {    // 全量
            String columnName = columnMap.get("columnName").toString();
            if (OLDColumnList.contains(columnName)) {
                OLDColumnList.remove(columnName);
            } else {
                insertList.add(columnMap);
            }
        }
        List<Map<String, Object>> deleteList = new ArrayList<>();
        for (String columnName : OLDColumnList) {
            // 通过columnName与权限ID获取到此行的数据
            deleteList.add(tempColumnSet.get(columnName));
        }

        if (deleteList.size() > 0) {
            String sql = " delete from querytemplatecolumnset where serviceOutPutId= ? and columnName=?";
            List<Object[]> selectParams = new ArrayList<>();
            for (Map<String, Object> deleteMap : deleteList) {
                List<String> inList = new ArrayList<>();
                inList.add(auth_id);
                inList.add(deleteMap.get("columnName").toString());
                selectParams.add(inList.toArray());
            }
            myDbHelper.updateByCondition(sql, selectParams);
        }
        String sql = "";
        if (insertList.size() > 0) {
            if (Objects.nonNull(tableName)) {
                sql = " insert into  querytemplatecolumnset(serviceOutPutId,columnName,columnLabel,columnDescribe) values(?,?,?,?)";
            } else {
                sql = " insert into  querytemplatecolumnset(serviceOutPutId,columnName,columnLabel) values(?,?,?)";
            }

            List<Object[]> insertParams = new ArrayList<>();
            for (Map<String, Object> insertMap : insertList) {
                List<Object> inList = new ArrayList<>();
                inList.add(auth_id);
                inList.add(insertMap.get("columnName").toString());
                inList.add(insertMap.get("columnLable").toString());
                if (Objects.nonNull(tableName)){
                    inList.add(insertMap.get("columnDescribe"));
                }
                insertParams.add(inList.toArray());
            }
            myDbHelper.updateByCondition(sql, insertParams);
        }
        return myDbHelper.queryByParamsReturnList("select * from querytemplatecolumnset where serviceOutPutId=?", auth_id);
    }

    //todo 目前采集实际是定时器，真正的采集是算法库中的算法，如果从头开始计算，代表是采集到的新数据，而不是旧数据的恢复
    // 如果需要真正的旧数据恢复，则必须从指定的算法开始，而不是从头开始
    // 需要将dataprocess对象中的算法循环，单独抽离出来，成为一个方法，此方法默认从第一个算法开始，也可以指定从第几个算法开始
    public Map<String, Object> abnormalDataRecovery(Map<String, Object> params) {
        String serviceId = Objects.isNull(params.get("serviceId")) ? null : params.get("serviceId").toString();
        Object dataObjectId = params.get("dataObjectId");
        if (MapTools.isBlank(serviceId) && Objects.isNull(dataObjectId)) {
            return processFail("serviceId与dataObjectId不能同时为空");
        }
        List<Object> pmList = new ArrayList<>();
        pmList.add("-1"); // todo 应该从logerror中获取最后一条，此时要求dataobject
        String sql = "select * from log_error_a where success = ? ";
        if (Objects.nonNull(serviceId)) {
            sql = sql + " and serviceId = ?";
            pmList.add(serviceId);
        }
        if (Objects.nonNull(dataObjectId)) {
            sql = sql + " and dataObjectId = ?";
            pmList.add(dataObjectId);
        }
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("异常数据恢复：获取远程数据库连接失败" + myDbHelper.getErrorMessage());
        }
        try {

            Map<String, Object> queryByParamsReturnListMap = myDbHelper.queryByParamsReturnList(sql, pmList.toArray());
            if (!queryByParamsReturnListMap.get("code").equals("0")) {
                return processFail("获远程日志错误： " + queryByParamsReturnListMap.get("message"));
            }
            List<Map<String, Object>> queryByParamsList = MapTools.getMapList(queryByParamsReturnListMap);
            if (Objects.isNull(queryByParamsList) || queryByParamsList.isEmpty()) {
                return processSuccess(null);
            }
            serviceId = queryByParamsList.get(0).get("serviceId").toString();
            DataProcess dataProcessObj = ObjectMap.getordropInput(serviceId);//获取数据输出处理对象
            if (null == dataProcessObj || !MapTools.isBlank(dataProcessObj.getErrorMessage())) {
                return processFail("服务不可用：" + (null == dataProcessObj ? "" : dataProcessObj.getErrorMessage()));
            }
            List<Object[]> deleteIds = new ArrayList<>();
            for (Map<String, Object> queryMap : queryByParamsList) {
                deleteIds.add(new ArrayList<String>() {{
                    add(queryMap.get("dataObjectId").toString());
                }}.toArray());
                Object iNDataContent = queryMap.get("iNDataContent");
                Object event = queryMap.get("event");
                if (Objects.nonNull(iNDataContent)) {
                    ObjectMapper objectMapper = new ObjectMapper();
                    List<Map<String, Object>> abnormalData = objectMapper.readValue(iNDataContent.toString(), List.class);
//                    dataProcessObj.execCalultion(abnormalData, queryMap.get("calculationLocation").toString(), event.toString(), queryMap.get("dataObjectId").toString(), null, null);
                }
            }
            myDbHelper.updateByCondition("delete  from log_error_a where dataObjectId = ?", deleteIds);
            return processSuccess(null);
        } catch (Exception e) {
            return processFail("异常数据恢复出现错误:" + LogUtils.getException(e));
        }
    }


    /**
     * 支持的最大机器id，结果是31 (这个移位算法可以很快计算出几位二进制数所能表示的最大十进制数)
     */
    private final String maxWorkerId = "4";
    /**
     * 支持的最大数据标识id，结果是31
     */
    private final String maxServiceId = "4";
    /**
     * 毫秒内序列(0~4095)
     */
    private final Long maxSequence = 999L;

    private long sequence = 0L;

    /**
     * 上次生成ID的时间截
     */
    private long lastTimestamp = -1L;


    public synchronized String createLifeCycleCol(Long workerId, Integer serviceId) {
        long timestamp = System.currentTimeMillis();
        //如果当前时间小于上一次ID生成的时间戳，说明系统时钟回退过这个时候应当抛出异常
        //如果是同一时间生成的，则进行毫秒内序列
        if (lastTimestamp == timestamp) {
            sequence++;
            //毫秒内序列溢出
            if (sequence > maxSequence) {
                //阻塞到下一个毫秒,获得新的时间戳
                sequence = 0;
                while (lastTimestamp == System.currentTimeMillis()) {
                }
                timestamp = System.currentTimeMillis();
            }
        }
        //时间戳改变，毫秒内序列重置
        else {
            sequence = 0L;
        }
        //上次生成ID的时间截
        lastTimestamp = timestamp;
        //移位并通过或运算拼到一起组成64位的ID
        return timestamp + "" + (String.format("%0" + (maxSequence.toString().length()) + "d", sequence)) + (String.format("%0" + maxWorkerId + "d", workerId)) + (String.format("%0" + maxServiceId + "d", serviceId));
    }
}
