package org.bfkj.application;

import org.bfkj.config.AppConfig;
import org.bfkj.config.ObjectMap;
import org.bfkj.utils.*;

import java.util.*;

/**
 * 服务输入控制类
 */
public class ServiceInputControl {
    public static Map<String, ScheduleTask> ScheduleTaskMaps = new HashMap<>();

    public static  Map<String, Object> processSuccess(Object returnData) {
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "0");
        returnMap.put("returnData", returnData);
        return returnMap;
    }

    //统一错误信息处理
    public static Map<String, Object> processFail(String errorMessage) {
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "-1");
        returnMap.put("message", errorMessage);
        return returnMap;
    }


    /**
     * 服务启动
     *
     * @param service_id
     * @return
     */
    public static Map<String,Object> start(String service_id) {
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("获取采集服务执行SQL异常");
        }
        //采集数据源及参数配置
        Map<String,Object> collectServiceMap = myDbHelper.queryByParamsReturnList("SELECT SI.* FROM  serviceinfo SI WHERE SI.serviceType ='4'  AND serviceID = ?", service_id);
        List<Map<String, Object>> collectServiceList = MapTools.getMapList(collectServiceMap);
        if (!collectServiceMap.get("code").equals("0")) {
            return processFail("获取采集服务执行SQL异常" + collectServiceMap.get("message"));
        }
        if (Objects.isNull(collectServiceList) || collectServiceList.isEmpty()) {
            return processFail("采集服务不存在");
        }
        Map<String, Object> collectServiceMaps = collectServiceList.get(0); //采集服务
        Integer loopCount = (null == collectServiceMaps.getOrDefault("loopCount", null)) ? null : Integer.parseInt(collectServiceMaps.get("loopCount").toString());
        int frequencyCount = (null == collectServiceMaps.getOrDefault("frequencyCount", null)) ? 1 : Integer.parseInt(collectServiceMaps.get("frequencyCount").toString());
        int frequencyUnit = (null == collectServiceMaps.getOrDefault("frequencyUnit", null)) ? 1 : Integer.parseInt(collectServiceMaps.get("frequencyUnit").toString());
        int taskType = (null == collectServiceMaps.getOrDefault("taskType", null)) ? 1 : Integer.parseInt(collectServiceMaps.get("taskType").toString());
        Object cronExpress = collectServiceMaps.get("cronExpress");
        Date taskValidDate = MapTools.strObjToDate(collectServiceMaps.get("taskValid"));
        Date taskInvalidDate = MapTools.strObjToDate(collectServiceMaps.get("taskInvalid"));
        //创建定时任务：
        ScheduleTask scheduleTask = ScheduleTaskMaps.get(service_id);
        if (Objects.isNull(scheduleTask)) {
            scheduleTask = new ScheduleTask(service_id, loopCount, taskValidDate, taskInvalidDate);
            ScheduleTaskMaps.put(service_id, scheduleTask);
        }
        // todo 记录日志
        boolean start = ScheduleUtil.start(scheduleTask, (frequencyCount * frequencyUnit), cronExpress, taskType);
        LogUtils.log("start: 2", "0", null, (start ? "启动成功" : "启动失败"), service_id,  null, null, null,null);//无条件记录数据接收日志
        if (start) {
            return processSuccess("启动成功");
        }
        myDbHelper.updateByCondition("update serviceinfo set runState = 1 where  serviceID =?",null,service_id);
        ScheduleTaskMaps.remove(service_id);
        return processFail("启动失败");
    }

    /**
     * 服务停止
     *
     * @param service_id
     * @return
     */
    public static Map<String,Object> stop(String service_id) {
        ScheduleTask scheduleTask = ScheduleTaskMaps.get(service_id);
        if (Objects.isNull(scheduleTask)) {
            return processFail("服务： " + service_id + " 已经停止");
        }
        boolean reset = ScheduleUtil.cancel(scheduleTask);
        ScheduleTaskMaps.remove(service_id);
        // 修改服务的状态
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("服务：" + service_id +" 停止 获取数据库对象失败，异常信息 " + myDbHelper.getErrorMessage());
        }
        myDbHelper.updateByCondition("update serviceinfo set runState = 0 where  serviceID =?",null,service_id);
        LogUtils.log("stop: 2", "0", null, (reset ? "停止成功" : "停止失败"), service_id, null, null, null,null);//无条件记录数据接收日志
        return processSuccess("停止" + (reset ? "成功" : "失败"));
    }

    public static Map<String,Object> startOrStop(Map<String, Object> params, String statue) {
        String serviceId = Objects.isNull(params.get("serviceId")) ? null : params.get("serviceId").toString();
        if (MapTools.isBlank(serviceId)) {
            return processFail("服务" + (statue.equals("1") ? "启动" : "停止") + (MapTools.isBlank(serviceId) ? "失败：serviceId为空" : "成功"));
        }

        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(AppConfig.REMOTE_DB_CONNECT);
        if (Objects.nonNull(myDbHelper.getErrorMessage())) {
            return processFail("startOrStop 获取数据库连接对象失败");

        }
        Map<String,Object> serviceTypeResult = myDbHelper.queryByParamsReturnList("select serviceType from serviceinfo where serviceID=?", serviceId);

        List<Map<String, Object>> serviceTypeData = MapTools.getMapList(serviceTypeResult);
        if (!serviceTypeResult.get("code").equals("0")) {
            return processFail("startOrStop 获取服务类型：执行sql异常" + serviceTypeResult.get("message"));
        }
        if (Objects.nonNull(serviceTypeData) && Objects.equals("4",serviceTypeData.get(0).get("serviceType").toString())) {
            if (statue.equals("0")) {
                return ServiceInputControl.stop(serviceId);
            }
            return ServiceInputControl.start(serviceId);
        }
        ObjectMap.getordropInput(serviceId,false);
        return processSuccess("重启成功");
    }
}
