package org.bfkj.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.ThreadPoolExecutor;

@EnableAsync
@Configuration
public class ThreadPoolConfig {
    @Value("${service.pool.maxPoolSize}")
    private Integer maxPoolSize;
    @Value("${service.pool.corePoolSize}")
    private Integer corePoolSize;
    @Value("${service.pool.queueCapacity}")
    private Integer queueCapacity; // #排队容量
    @Value("${service.pool.keepAliveSeconds}")
    private Integer keepAliveSeconds;
    @Value("${service.pool.threadNamePrefix}")
    private String threadNamePrefix;
    @Value("${service.pool.waitForTasksToCompleteOnShutdown}")
    private Boolean waitForTasksToCompleteOnShutdown;

    @Bean("ThreadPoolTaskExecutor")
    public ThreadPoolTaskExecutor asyncServiceExecutor() {
        ThreadPoolTaskExecutor outExecutor = new ThreadPoolTaskExecutor();
        // 设置核心线程数等于系统核数--8核
        int availableProcessors = Runtime.getRuntime().availableProcessors();
        outExecutor.setCorePoolSize(availableProcessors);
        // 设置最大线程数
        outExecutor.setMaxPoolSize(maxPoolSize);
        outExecutor.setCorePoolSize(corePoolSize);
        //配置队列大小
        outExecutor.setQueueCapacity(queueCapacity);
        // 设置线程活跃时间（秒）
        outExecutor.setKeepAliveSeconds(keepAliveSeconds);
        // 线程满了之后由调用者所在的线程来执行
        // 拒绝策略：CALLER_RUNS：不在新线程中执行任务，而是由调用者所在的线程来执行
        outExecutor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        // 设置默认线程名称
        outExecutor.setThreadNamePrefix(threadNamePrefix);
        // 等待所有任务结束后再关闭线程池
        outExecutor.setWaitForTasksToCompleteOnShutdown(waitForTasksToCompleteOnShutdown);
        //执行初始化
        outExecutor.initialize();
        return outExecutor;
    }
}
