package org.bfkj.protocol;

import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.bfkj.utils.LogUtils;
import org.bfkj.utils.MapTools;
import org.elasticsearch.action.ShardOperationFailedException;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.action.support.WriteRequest;
import org.elasticsearch.action.support.replication.ReplicationResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestClientBuilder;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.xcontent.XContentType;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/*es协议*/
public class ElasticHandler {

    private static RestHighLevelClient restHighLevelClient;

    public boolean sendData(String indexName, List<Object> dataContent, String connectConfig, String serviceId) {
        if (Objects.isNull(indexName)) {
            MapTools.processFail("队列为空: " + dataContent);
            return true;
        }
        if (Objects.isNull(dataContent)) {
            MapTools.processSuccess(null);
            return true;
        }

        if (Objects.isNull(restHighLevelClient) ){
            try {
                Map<String, Object> connectConfigMaps = (Map<String, Object>) MapTools.strToObj(connectConfig);
                String scheme = connectConfigMaps.get("scheme").toString();
                String hostname = connectConfigMaps.get("hostname").toString();
                int port = Integer.parseInt(connectConfigMaps.get("port").toString());
                Object account = connectConfigMaps.get("auth");
                if (null != account && "true".equals(account.toString())) {
                    //有权限验证
                    String username = connectConfigMaps.get("username").toString();
                    String password = connectConfigMaps.get("password").toString();
                    final CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
                    credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(username, password));
                    RestClientBuilder restClientBuilder = RestClient.builder(new HttpHost(hostname, port, scheme)).setHttpClientConfigCallback((httpAsyncClientBuilder) -> httpAsyncClientBuilder.setDefaultCredentialsProvider(credentialsProvider));
                    restHighLevelClient = new RestHighLevelClient(restClientBuilder);
                } else {
                    //无权限验证
                    restHighLevelClient = new RestHighLevelClient(RestClient.builder(new HttpHost(hostname, port, scheme)));
                }
            } catch (Exception e) {
                MapTools.processFail("es发送参数异常：" + LogUtils.getException(e));
                return false;
            }
        }
        String dataJSON = "";
        try {
            if (dataContent.size() > 0) {
                IndexRequest request = new IndexRequest(indexName);
                for (Object data : dataContent) {
                    dataJSON = MapTools.objToJSONStr(data);
                    request.id(createLifeCycleCol(1L, Integer.valueOf(MapTools.isBlank(serviceId) ? "0" : serviceId)));
                    request.timeout("1s");
                    request.source(dataJSON, XContentType.JSON);
                    request.setRefreshPolicy(WriteRequest.RefreshPolicy.IMMEDIATE);
                    IndexResponse indexResponse = restHighLevelClient.index(request, RequestOptions.DEFAULT);
                    ReplicationResponse.ShardInfo shardInfo = indexResponse.getShardInfo();
                    if (shardInfo.getFailed() > 0) {
                        String collect = Arrays.stream(shardInfo.getFailures()).map(ShardOperationFailedException::reason).collect(Collectors.joining(";\n"));
                        MapTools.processSuccess(collect);
                        return false;
                    }
                }
            }
        } catch (IOException e) {
            try {
                restHighLevelClient.close();
            } catch (IOException ex) {
                System.out.println("es数据发送异常： ".concat(LogUtils.getException(e)));
            }
            try {
                Thread.sleep(1000 * 6);
            } catch (InterruptedException ex) {
                System.out.println("es数据发送异常： ".concat(LogUtils.getException(e)));
            }
            restHighLevelClient = null;
            return  false;
        }
        return true;
    }

    private long sequence = 0L;
    private long lastTimestamp = -1L;

    public String createLifeCycleCol(Long workerId, Integer serviceId) {
        long timestamp = System.currentTimeMillis();
        //如果当前时间小于上一次ID生成的时间戳，说明系统时钟回退过这个时候应当抛出异常
        if (lastTimestamp == timestamp) { //如果是同一时间生成的，则进行毫秒内序列
            sequence++;
            if (sequence > 999L) {//毫秒内序列溢出
                sequence = 0;
                while (lastTimestamp == System.currentTimeMillis()) {//阻塞到下一个毫秒,获得新的时间戳
                }
                timestamp = System.currentTimeMillis();
            }
        } else {
            sequence = 0L;
        }
        lastTimestamp = timestamp;//上次生成ID的时间截
        //移位并通过或运算拼到一起组成64位的ID
        return String.valueOf(timestamp).concat(String.format("%03d", sequence)).concat(String.format("%04d", workerId)).concat(String.format("%04d", serviceId));
    }
}
