package org.bfkj.protocol;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.bfkj.utils.MapTools;

import java.time.Duration;
import java.util.*;

public class MyKafKa {

    private KafkaConsumer<String, Object> consumer;
    private KafkaProducer<String, Object> producer;

    private boolean isPool =false;

    private static ObjectMapper mapper = new ObjectMapper();
    static {
        mapper.registerModule(new JavaTimeModule());
        mapper.configure(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS, false); //关闭
    }

    /**
     * 采集kafka数据
     *
     * @return 读取内容信息
     */
    public Map<String, Object> readMethod(String sourceObjectName, String library_id, String connectConfig, String  pollNumber,String fetch) {
        if (Objects.isNull(consumer)) {
            if (Objects.isNull(sourceObjectName)) {
                return MapTools.processFail("队列为空");
            }
            Map<String, Object>  connectConfigMaps = (Map<String, Object>) MapTools.strToObj(connectConfig);
            if (!connectConfigMaps.containsKey("group.id")) {
                connectConfigMaps.put("group.id", "groupid" + (String.format("%s", library_id)));
            }
            connectConfigMaps.put("max.poll.records", pollNumber == null ? 1 : Integer.parseInt(pollNumber));
            connectConfigMaps.put("key.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
            connectConfigMaps.put("value.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
            if (MapTools.isNotBlank(fetch)){
                connectConfigMaps.put("max.partition.fetch.bytes", 11534336);
            }
            consumer = new KafkaConsumer<>(connectConfigMaps);
            consumer.subscribe(Collections.singleton(sourceObjectName));
        }
//        System.out.println(consumer +"   library_id: " + library_id);
        //todo 1. 如果一次行读取多条 2.偏移修改  : // Duration.ofSeconds(1): 如果没有数据则此处会等待1秒
        isPool = true;
        ConsumerRecords<String, Object> records = consumer.poll(Duration.ofSeconds(1));
        List<String> messageList = new ArrayList<>();
        if (records.count() > 0) {
            for (ConsumerRecord<String, Object> record : records.records(sourceObjectName)) {
                String readValue = record.value().toString().trim();
                if (MapTools.isBlank(readValue) || readValue.equals("{}")){
                    continue;
                }
                messageList.add(readValue);
            }
        }
        isPool =false;
        return MapTools.processSuccess(messageList);
    }

    /**
     * 数据发送到kafka
     *
     * @return 发送结果信息
     */
    public Map<String, Object> sendMethod(String sourceObjectName, List<Object> dataContent, String connectConfig) {
        if (Objects.isNull(sourceObjectName)) {
            return MapTools.processFail("队列为空: " + dataContent);
        }
        if (Objects.isNull(dataContent)) {
            return MapTools.processSuccess(null);
        }
        Map<String, Object>  connectConfigMaps = (Map<String, Object>) MapTools.strToObj(connectConfig);
        connectConfigMaps.put("request.timeout.ms", 60000);
        connectConfigMaps.put("max.request.size", 10240000);
        connectConfigMaps.put("compression.type", "gzip");
        connectConfigMaps.put("key.serializer", "org.apache.kafka.common.serialization.StringSerializer");
        connectConfigMaps.put("value.serializer", "org.apache.kafka.common.serialization.StringSerializer");

        if (Objects.isNull(producer)) producer = new KafkaProducer<>(connectConfigMaps);
        List<Object> sendResult = new ArrayList<>();
        for (Object o : dataContent) {
            try {
                if(o instanceof String && MapTools.isBlank(o.toString().trim()) ){
                    continue;
                }
                sendResult.add(producer.send(new ProducerRecord<>(sourceObjectName, mapper.writeValueAsString(o))));
            } catch (Exception e) {
                sendResult.add(producer.send(new ProducerRecord<>(sourceObjectName, o.toString())));
            }
        }
        return MapTools.processSuccess(sendResult);
    }

    /**
     * 关闭kafka信息
     *
     * @throws Exception
     */
    public void close() {
        while (isPool){
            try {
                Thread.sleep(1);
            } catch (InterruptedException e) {
            }
        }
        if (Objects.nonNull(consumer)) {
            consumer.close();
            consumer = null;
            System.out.println("consumer关闭");
        }

        if (Objects.nonNull(producer)) {
            producer.close();
            producer = null;
            System.out.println("consumer关闭");
        }
    }

}
