package org.bfkj.protocol;

import org.bfkj.utils.LogUtils;
import org.bfkj.utils.MapTools;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.web.client.RestTemplate;

import java.nio.charset.StandardCharsets;
import java.util.*;

public class WebAPI {

    private RestTemplate restTemplate; //webapi请求对象
    private String webapiURL;

    private HttpHeaders webApiHeader;
    private String webapiMethod;

    private Map<String, Object> baseHeader = new HashMap<>();

    /**
     * webapi 执行入口函数
     */
    public Map<String, Object> execWebApi(Object header, String method, Object defaultBody, String connectConfig) {
        Map<String, Object> connectConfigMaps = (Map<String, Object>) MapTools.strToObj(connectConfig);
        Map<String, Object> restTemplateResult = initWebApiParams(connectConfigMaps); // 通过连接配置 获取restTemplate

        if (!"0".equals(restTemplateResult.get("code"))) {
            return MapTools.processFail(restTemplateResult.get("message").toString());
        }
        if (header instanceof Map) {
            Map<String, Object> temeMap = (Map) header;
            if (!temeMap.isEmpty()) {
                temeMap.putAll(baseHeader);
                webApiHeader.clear();
                for (String key : temeMap.keySet()) {
                    webApiHeader.add(key, temeMap.get(key).toString());
                }
            }
        }
        if (MapTools.isNotBlank(method)) webapiMethod = method.toLowerCase();
        try {
            List<Map<String, Object>> returnList = new ArrayList<>();
            if (defaultBody instanceof List<?> tempList) {
                for (Object o : tempList) {
                    returnList.add(sendWebApiRequest(o));
                }
            } else {
                returnList.add(sendWebApiRequest(defaultBody));
            }
            return (defaultBody instanceof List<?>) ? MapTools.processSuccess(returnList) : returnList.get(0);
        } catch (Exception e) {
            return MapTools.processFail("restTemplate执行异常" + LogUtils.getException(e));
        }
    }

    private Map<String, Object> sendWebApiRequest(Object defaultBody) {
        try {
            HttpEntity<Object> request = new HttpEntity<>(defaultBody, webApiHeader);
            String responseEntity = null;
            switch (webapiMethod) {
                case "get" -> responseEntity = restTemplate.getForObject(webapiURL, String.class, request);
                case "put" -> restTemplate.put(webapiURL, request);
                case "delete" -> restTemplate.delete(webapiURL, request);
                default -> responseEntity = restTemplate.postForObject(webapiURL, request, String.class);
            }
            if (responseEntity == null && (webapiMethod.equals("put") || webapiMethod.equals("delete"))) {
                return MapTools.processSuccess(null);
            }
            if (Objects.nonNull(responseEntity)) {
                return MapTools.processSuccess(responseEntity);
            } else {
                return MapTools.processFail("webapi发送请求失败 : " + responseEntity);
            }
        }catch (Exception e){
            return MapTools.processFail("发送请求失败 : ".concat(LogUtils.getException(e)));
        }

    }

    public Map<String, Object> initWebApiParams(Map<String, Object> connectConfig) {
        try {
            if (restTemplate == null) {
                if (Objects.isNull(connectConfig)) {
                    return MapTools.processFail("webapi请求配置参数为空");
                }
                webapiURL = Objects.nonNull(connectConfig.get("url")) ? connectConfig.get("url").toString() : null;
                if (Objects.isNull(webapiURL)) {
                    return MapTools.processFail("webapi请求地址为空");
                }
                Object tempHeaders = connectConfig.get("headers"); //需确定返回的是一个map对象还是字符串  headers: {ContentType:"JSON"}
                webApiHeader = new HttpHeaders();
                if (Objects.isNull(tempHeaders)) {
                    webApiHeader.setContentType(MediaType.APPLICATION_JSON);
                }
                if (tempHeaders instanceof Map) {
                    baseHeader = (Map) tempHeaders;
                    if (!baseHeader.isEmpty()) {
                        for (String key : baseHeader.keySet()) {
                            webApiHeader.add(key, baseHeader.get(key).toString());
                        }
                    }
                }
                Object webMethod = connectConfig.get("method"); //需确定返回的是一个map对象还是字符串  headers: {ContentType:"JSON"}
                if (Objects.nonNull(webMethod)) {
                    webapiMethod = webMethod.toString().toLowerCase();
                } else {
                    webapiMethod = "post";
                }
//            HttpComponentsClientHttpRequestFactory httpComponentsClientHttpRequestFactory = new HttpComponentsClientHttpRequestFactory();
//            httpComponentsClientHttpRequestFactory.setConnectTimeout(6000);
//            httpComponentsClientHttpRequestFactory.setConnectionRequestTimeout(6000);
                restTemplate = new RestTemplate();
                restTemplate.getMessageConverters().set(1, new StringHttpMessageConverter(StandardCharsets.UTF_8));
            }
        } catch (Exception e) {
                   return MapTools.processFail(LogUtils.getException(e));
        }
        return MapTools.processSuccess(null);
    }

    /**
     * 关闭kafka信息
     *
     * @throws Exception
     */
    public void close() {
        if (Objects.nonNull(restTemplate)) {
            restTemplate = null;
            System.out.println("restTemplate关闭");
        }
    }
}
