package org.bfkj.utils;


import org.bfkj.config.AppConfig;
import org.bfkj.config.ObjectMap;
import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 数据格式转换
 * <p>
 * <p>
 * 1. 添加多线程
 * 2. 数据项结构项表
 */
public class DataFormat {
    private static Map<String, String> messageRules = new HashMap<>();
    private static Map<String, String> EMonth = new HashMap<>();
    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    static {
        messageRules.put("BSM", "BSM[\\s\\S]+?ENDBSM");
        messageRules.put("BPM", "BPM[\\s\\S]+?ENDBPM");
        EMonth.put("JAN", "01");
        EMonth.put("FEB", "02");
        EMonth.put("MAR", "03");
        EMonth.put("APR", "04");
        EMonth.put("MAY", "05");
        EMonth.put("JUN", "06");
        EMonth.put("JUL", "07");
        EMonth.put("AUG", "08");
        EMonth.put("SEP", "09");
        EMonth.put("OCT", "10");
        EMonth.put("NOV", "11");
        EMonth.put("DEC", "12");
    }

    public static Map<String, Object> processSuccess(Object returnData) {
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "0");
        returnMap.put("returnData", returnData);
        return returnMap;
    }

    //统一错误信息处理
    public static Map<String, Object> processFail(String errorMessage) {
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "-1");
        returnMap.put("message", errorMessage);
        return returnMap;
    }

    /**
     * base64解码
     *
     * @param source 原已编码数据
     * @return 原始数据
     */
    public static String base64DecoderToString(String source) {
        return new String(Base64.getDecoder().decode(source));
    }

    /**
     * base64 编码
     *
     * @param source 原数据
     * @return 编码后数据
     */
    public static String base64EncoderToString(String source) {
        return Base64.getEncoder().encodeToString(source.getBytes());
    }

    /**
     * @param str
     * @return
     */
    private static String replaceWrap(String str) {
        int rIndex = str.indexOf("\r");
        int nIndex = str.indexOf("\n");
        int index = str.indexOf("\\");
        int tempIndex = str.length();
        if (rIndex > 0) {
            tempIndex = rIndex;
        }
        if (nIndex > 0 && nIndex < tempIndex) {
            tempIndex = nIndex;
        }
        if (index > 0 && index < tempIndex) {
            tempIndex = index;
        }
        return tempIndex == str.length() ? str : str.substring(0, tempIndex);
    }

    public static Map<String, Object> typeBToMap(Object typeStr, String dataObjectId, Map<String, String> transMaps) {
        try {
            if (Objects.isNull(typeStr)) return MapTools.processSuccess(null);
            if (Objects.isNull(transMaps)) transMaps = new HashMap<>();
            List<Object> tempList = new ArrayList<>();
            if (typeStr instanceof List<?> tempInList) {
                tempList.addAll(tempInList);
            } else {
                tempList.add(typeStr);
            }
            List<Map<String, Object>> rsList = new ArrayList<>();
            for (Object o : tempList) {
                String inData = Objects.toString(o);
                if (MapTools.isBlank(inData)) continue;
                int dataObjectIdIndex = inData.indexOf("dataObjectId");
                String tpDataObjectId = inData.length() > 39 ? inData.substring(dataObjectIdIndex + 15, dataObjectIdIndex + 39) : null;
                rsList.addAll(typebHandler(inData, transMaps, MapTools.isBlank(tpDataObjectId) ? dataObjectId : tpDataObjectId));
            }
            return processSuccess(rsList);
        } catch (Exception e) {
            return processFail(LogUtils.getException(e));
        }
    }


    public static List<Map<String, Object>> typebHandler(String typeStr, Map<String, String> transMaps, String dataObjectId) {
        List<String> typeStrList = MapTools.patternContent(typeStr, "\\{(.+?)\\}");
        //设定报文提取规则MAP，此MAP应该放到类变量中，属于常量，没必要每次调用都去创建：未来从配置文件或者数据库读取TypeB的所有数据格式，只是目前只支持BSM和BPM，未来可以加
        List<Map<String, Object>> returnDataList = new ArrayList<>();//初始化一个最终返回的数组
        for (String dataType : messageRules.keySet()) {//循环报文提取规则MAP:为了兼容一条字符串中既有BSM又有BPM的情况
            List<String> messagList = new ArrayList<>();
            for (String tempTypeStr : typeStrList) {
                if (tempTypeStr.indexOf("dataObjectId") > 0) {
                    String tempDataObjectId = tempTypeStr.substring(tempTypeStr.indexOf("dataObjectId"));
                    dataObjectId = tempDataObjectId.replaceAll("[^0-9]+", "");
                }
                List<String> tempMessageList = MapTools.patternContent(tempTypeStr, messageRules.get(dataType)); //使用对应的提取正则表达式进行报文提取
                if (tempMessageList.size() > 0) {
                    messagList.addAll(tempMessageList);
                } else {
                    if (tempTypeStr.contains(dataType)) {
                        HashMap<String, Object> errorMap = new HashMap<>();
                        errorMap.put("dataObjectId", dataObjectId);
                        errorMap.put("errMessage", "报文数据格式错误");
                        if ("true".equals(AppConfig.SOURCE_DATA)){
                            errorMap.put("sourceData", tempTypeStr);
                        }
                        if (!returnDataList.contains(errorMap)) {
                            returnDataList.add(errorMap);
                        }
                    }
                }
            }
            for (String signData : messagList) {//循环提取到的报文数组中每一条报文
                Map<String, Object> signDataMap = new HashMap<>();//创建一个当前报文的MAP
                signDataMap.put("dataType", dataType);//默认报文类型
                signDataMap.put("dataObjectId", dataObjectId); //默认生命周期ID
//                signDataMap.put("sourceData", signData); //默认原始报文--因为要记录完整的原始报文，所以不去头不去尾
                if ("true".equals(AppConfig.SOURCE_DATA)){
                    signDataMap.put("sourceData", signData);
                }
                String[] rowDataList = signData.split("(?=\\.[A-Z]{1}\\/)", -1);//使用.字母/进行分组，获得行数据
                //获取第一行的数据：第一行,BPM为空，BSM直接使用indexOf判断即可
                signDataMap.put("dataState", dataType.equals("BSM") ? (rowDataList[0].contains("DEL") ? "DEL" : (rowDataList[0].contains("CHG") ? "CHG" : "ADD")) : null);
                List<Map<String, Object>> containerList = new ArrayList<>();//容器数组
                List<Map<String, Object>> luggageList = new ArrayList<>();//行李数组
                for (int rowNum = 1; rowNum < rowDataList.length; rowNum++) {  //循环行数据
                    String rowDataStr = rowDataList[rowNum];//获取单行数据
                    String rowType = rowDataStr.substring(0, 3);//头三位是行标识
                    if (".B/.N/".contains(rowType)) {//如果是.B\.N开头则//luggageList = getluggageNumList(rowDataStr);//--单独一个方法处理
                        List<String> luggageNumList = new ArrayList<>();//默认行李牌号数组为空数组
                        String luggageState = null;//默认行李状态为NULL
                        String[] colList = rowDataStr.split("\\/", -1);//使用\进行分组，获得列数据
                        for (int colNum = 1; colNum < colList.length; colNum++) {//循环
                            String colValue = colList[colNum].trim();//去空格 OFF,3666149457002
                            colValue = replaceWrap(colValue);
                            if (MapTools.isNumber(colValue) && colValue.length() == 13) {//如果是13位数字则
                                String luggageNum = colValue.substring(0, 10);////获取前10位--开始行李牌号
                                for (int loopNum = 0; loopNum < Integer.parseInt(colValue.substring(10, 13)); loopNum++) {//循环连续行李数
                                    luggageNum = String.format("%010d", (Long.parseLong(luggageNum) + loopNum)); //计算行李牌号，注意不足10位时前面补0：0999123456变数字后是9位
                                    luggageNumList.add(luggageNum);//添加到行李牌号数组 3666149457,3666149458
                                }
                            } else {//
                                luggageState = MapTools.isNotBlank(colValue) ? colValue.substring(0, 3) : null;//更新行李状态
                            }
                        }
                        for (String luggageNum : luggageNumList) {//循环行李牌号数组
                            Map<String, Object> luggageMap = new HashMap<>();
                            luggageMap.put("luggageNum", luggageNum);
                            luggageMap.put("abnormalState", luggageState);
                            luggageList.add(luggageMap);//追加到行李数组中{行李牌号,行李状态}
                        }
                    }
                    //如果是.C\.E\.L\.M\.P\.Q\.R\.Q\.T开头则//去头写入当前行MAP{"对应数据库字段名",值（,号分割的字符串）
                    if (".C/.E/.L/.M/.P/.Q/.R/.T/".contains(rowType)) {
                        rowDataStr = rowDataStr.replace(rowType, "").trim();//去头去空格
//                            rowDataStr = rowDataStr.indexOf("\\") > 0 ? rowDataStr.substring(0, rowDataStr.indexOf("\\") - 1) : rowDataStr;//去换行符
                        rowDataStr = replaceWrap(rowDataStr);
                        if (".P/".equals(rowType)) {//姓名需要去掉首位数字和分割符
                            rowDataStr = rowDataStr.replaceAll("[(0-9)]", "").replaceAll("\\/", "");
                        }
                        if (Objects.nonNull(transMaps.get(rowType))) {
                            signDataMap.put(transMaps.get(rowType), (signDataMap.containsKey(rowType) ? signDataMap.get(rowType) + "," : "") + rowDataStr);
                        }
                    }
                    //如果是.D\.F\.G\.H\.I\.J\.O\.S\.W\.X\.Y开头则只记录第一个，存在则不覆盖
                    if (".D/.F/.G/.H/.I/.J/.O/.S/.W/.X/.Y/".contains(rowType) && !signDataMap.containsKey(rowType + "1")) {
                        String[] colList = rowDataStr.split("\\/", -1);//使用\进行分组，获得列数据
                        for (int colNum = 1; colNum < colList.length; colNum++) {//循环
                            String colValue = colList[colNum].trim();//去空格
                            colValue = replaceWrap(colValue);
                            //如果是日期则使用日期转换方法进行转换（08APR）（注意跨年处理）".F/"".I/"".O/"有日期，".D/"".J/"有日期和时间
                            if (".F/.I/.O/.D/.J/".contains(rowType)) {
                                String patternValue = MapTools.patternKey(colValue, "[0-3]{1}[0-9]{1}(JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC)");
                                if (!MapTools.isBlank(patternValue)) {
                                    colValue = TypeBDateChange(patternValue, null);
                                    signDataMap.put(rowType.equals(".D/") ? ".D/3" : (rowType.equals(".J/") ? ".J/4" : (rowType + "2")), colValue);//写入当前行MAP{"对应数据库字段名",值（不覆盖）}
                                }
                                if (".F/.I/.O/".contains(rowType) && MapTools.isNotBlank(colValue)) {
                                    if (colValue.length() == 3 && !MapTools.isBlank(MapTools.patternKey(colValue, "[A-z]{3}"))) {
                                        if (!signDataMap.containsKey(transMaps.get(rowType + "3"))) {
                                            signDataMap.put(transMaps.get(rowType + "3"), colValue);//写入当前行MAP{"对应数据库字段名",值（不覆盖）}
                                        }
                                    }
                                    if (colNum == 1) {
                                        String temp = colValue.substring(2);
                                        signDataMap.put(transMaps.get(rowType + "1"), colValue.substring(0, 2) + ((MapTools.isNumber(temp.substring(temp.length() - 1)) ? String.format("%4s", temp) : String.format("%5s", temp)).replace(" ", "0")));//写入当前行MAP{"对应数据库字段名",值（不覆盖）}
                                    }
                                }
                                if ((rowType + "" + colNum).equals(".J/5") || (rowType + "" + colNum).equals(".D/4")) {
                                    colValue = TypeBDateChange(null, colValue);
                                }
                            }
                            if (".S/".contains(rowType) && MapTools.isNotBlank(colValue)) {
                                if ("I,V,A".contains(colValue) && colValue.length() == 1 && !signDataMap.containsKey(transMaps.get(".S/8"))) {
                                    signDataMap.put(transMaps.get(".S/8"), colValue);//写入当前行MAP{"对应数据库字段名",值（不覆盖）}
                                }
                            }
                            if (".X/".contains(rowType) && MapTools.isNotBlank(colValue)) {
                                if (colValue.length() == 3) {
                                    if ("CLR,RFJ,UCL".contains(colValue)) {
                                        signDataMap.put(transMaps.get(".X/2"), colValue);//写入当前行MAP{"对应数据库字段名",值（不覆盖）}
                                    } else {
                                        signDataMap.put(transMaps.get(".X/1"), colValue);//写入当前行MAP{"对应数据库字段名",值（不覆盖）}
                                    }
                                }
                            }
                            //如果是时间则使用时间转换方法进行转换转换（082512;082512L;082512Z,0825;0825L;0825Z）（注意没有结尾字母则是）".D/"".J/"有日期和时间
                            //如果是日期时间则使用日期时间转换方法进行转换转换".G/"有日期时间（25NOV1600）
                            //如果同时存在日期和时间".D/"".J/"有日期和时间
                            String key = rowType + "" + colNum;
                            if (".D/3.D/4.J/4.J/5".contains(rowType + "" + colNum)) {
                                if (!signDataMap.containsKey(key)) {
                                    signDataMap.put(rowType + "" + colNum, colValue);//写入当前行MAP{"对应数据库字段名",值（不覆盖）}
                                }
                            } else {
                                if (Objects.nonNull(transMaps.get(rowType + "" + colNum))) {
                                    if (!signDataMap.containsKey(transMaps.get(key))) {
                                        signDataMap.put(transMaps.get(rowType + "" + colNum), colValue);//写入当前行MAP{"对应数据库字段名",值（不覆盖）}
                                    }
                                }
                            }

                        }
                        // 处理D3D4 //J4J5
//                            "D3D4": "checkInDate",
                        if (Objects.nonNull(signDataMap.get(".D/3")) && Objects.nonNull(signDataMap.get(".D/4"))) {
                            String d4 = signDataMap.get(".D/4").toString();
                            signDataMap.put("checkInTimeType", d4.substring(d4.length() - 1));
                            signDataMap.put(transMaps.get(".D/3.D/4"), (signDataMap.get(".D/3") + " " + d4.substring(0, d4.length() - 1)).trim());
                            signDataMap.remove(".D/3");
                            signDataMap.remove(".D/4");
                        }
                        if (Objects.nonNull(signDataMap.get(".J/4")) && Objects.nonNull(signDataMap.get(".J/5"))) {
                            String J5 = signDataMap.get(".J/5").toString();
                            signDataMap.put("processTimeType", J5.substring(J5.length() - 1));
                            signDataMap.put(transMaps.get(".J/4.J/5"), (signDataMap.get(".J/4") + " " + J5.substring(0, J5.length() - 1)).trim());
                            signDataMap.remove(".J/4");
                            signDataMap.remove(".J/5");
                        }
                    }
                    if (".V/".equals(rowType) && !signDataMap.containsKey(rowType + "1")) {//存在则不覆盖
                        signDataMap.put(transMaps.get(".V/1"), rowDataStr.substring(4, 5));//写入当前行MAP{"来源标识",值}
                        signDataMap.put(transMaps.get(".V/2"), rowDataStr.substring(5, 8));//写入当前行MAP{"来源机场",值}
                    }
                    if (".U/".equals(rowType)) {//如果是.U开头则----出现两个.U的需要单独处理
                        Map<String, Object> containerMap = new HashMap<>();//创建一个当前报文的MAP
                        String[] colList = rowDataStr.split("\\/", -1);//使用\进行分组，获得列数据
                        for (int colNum = 1; colNum < colList.length; colNum++) {//循环
                            String colValue = colList[colNum];//去空格
                            colValue = replaceWrap(colValue);
                            if (Objects.nonNull(transMaps.get(rowType + "" + colNum))) {
                                containerMap.put(transMaps.get(rowType + "" + colNum), colValue);//写入容器MAP
                            }

                        }
                        containerList.add(containerMap);//添加到容器数组中
                    }
                }

                //处理容器以及行李数组
                if (containerList.size() == 0) {//为0时添加空MAP，方便下面的循环
                    containerList.add(new HashMap<>());
                }
                if (containerList.size() > 2) {//最多两个容器，否则是异常数据，整个报文都作废
                    signDataMap.put("errMessage", "容器数量超规 " + containerList.size());//只需要标记错误，因为已经有原始报文存在
                    Map<String, Object> containerMap = containerList.get(0);
                    containerList = new ArrayList<>() {{
                        add(containerMap);
                    }};
//                        returnDataList.add(signDataMap);//添加到返回数组中
                }
                for (int containerNum = 0; containerNum < containerList.size(); containerNum++) {//循环容器数组
                    HashMap<String, Object> tempDataMapClone = MapTools.clone(signDataMap);
                    HashMap<String, Object> tempDataMap = Objects.isNull(tempDataMapClone) ? new HashMap<>() : tempDataMapClone;  //深拷贝添加到单条报文的MAP中
                    Map<String, Object> containerMap = containerList.get(containerNum);//获取当前容器数据
                    if (containerNum == 0 && containerList.size() > 1) {//如果当前是第一条且容器数组大于代表第一个容器是卸下，第二个容器是
                        containerMap.put("containerState", "OFF");//添加容器状态为OFF，代表卸下
                    }
                    if (!containerMap.isEmpty()) {//如果当前容器数据不是空
                        tempDataMap.putAll(containerMap);//深拷贝添加到单条报文的MAP中
                    }
                    tempDataMap.values().removeAll(Collections.singleton(null));
                    tempDataMap.values().removeAll(Collections.singleton(""));
                    if (luggageList.size() == 0) {
                        returnDataList.add(specialHandler(tempDataMap, dataObjectId)); //只有容器没有行李:整箱操作
                    } else {
                        for (Map<String, Object> tempLuggageMap : luggageList) {//循环行李数组
                            if (containerNum == 0 && containerList.size() > 1 && tempLuggageMap.get("abnormalState") == null) {//只添加.B行李
                                continue;
                            }
                            if (containerNum != 0 && containerList.size() > 1 && tempLuggageMap.get("abnormalState") != null) {//只添加.N行李
                                continue;
                            }
                            HashMap<String, Object> objectObjectHashMap = new HashMap<>();
                            objectObjectHashMap.putAll(tempDataMap);
                            objectObjectHashMap.putAll(tempLuggageMap);

                            if (Objects.isNull(objectObjectHashMap.get("abnormalState"))) {
                                objectObjectHashMap.remove("abnormalState");
                            }
                            returnDataList.add(specialHandler(objectObjectHashMap, dataObjectId));//添加到返回数组中
                        }
                    }
                }
            }
        }
        if (returnDataList.size() == 0) {
            HashMap<String, Object> errorMap = new HashMap<>();
            errorMap.put("dataObjectId", dataObjectId);
            errorMap.put("errMessage", "报文数据格式错误");
            if ("true".equals(AppConfig.SOURCE_DATA)){
                errorMap.put("sourceData", typeStr);
            }
            returnDataList.add(errorMap);
        }
        return returnDataList;
    }

    /**
     * //必要的特殊处理：
     * //承运航班：如果有F则F否则如果有I则I否则空
     * //起飞机场：如果有F则V否则如果有I则I否则空
     * //降落机场：如果有F则F否则V
     * //读取地点：如果有J则J否则如果有D则D否则有H则H
     * //读取时间：如果有J则J否则如果有D则D否则接收时间
     * //当前结果：如果有dataState则dataState否则有B则B否则有X则X
     *
     * @return
     */
    public static Map<String, Object> specialHandler(HashMap<String, Object> inMap, String dataObjectId) {
        String outAirport = inMap.containsKey("flightNo") ? "sourceAirport" : (inMap.containsKey("inairport") ? "inairport" : null);
        if (MapTools.isNotBlank(outAirport)) {
            inMap.put("outAirport", inMap.get(outAirport));
        }
        String landAirport = inMap.containsKey("target_airport") ? "target_airport" : (inMap.containsKey("sourceAirport") ? "sourceAirport" : null);
        if (MapTools.isNotBlank(landAirport)) {
            inMap.put("landAirport", inMap.get(landAirport));
        }

        String carrierFlights = inMap.containsKey("flightNo") ? "flightNo" : (inMap.containsKey("inflightNo") ? "inflightNo" : null);
        if (MapTools.isNotBlank(carrierFlights)) {
            inMap.put("carrierFlights", inMap.get(carrierFlights));
        }
        String carrierFlightsDate = inMap.containsKey("flightDate") ? "flightDate" : (inMap.containsKey("inflightDate") ? "inflightDate" : null);
        if (MapTools.isNotBlank(carrierFlightsDate)) {
            inMap.put("carrierFlightsDate", inMap.get(carrierFlightsDate));
        }

        String readAddess = inMap.containsKey("location_code") ? "location_code" : (inMap.containsKey("checkInLocation") ? "checkInLocation" : inMap.containsKey("operationArea") ? "operationArea" : (inMap.containsKey("location_describe") ? "location_describe" : null));
        if (MapTools.isNotBlank(readAddess)) {
            inMap.put("readAddess", inMap.get(readAddess));
        }
        String readTime = inMap.containsKey("processing_time") ? "processing_time" : (inMap.containsKey("checkInDate") ? "checkInDate" : null);
        inMap.put("readTime", MapTools.isBlank(readTime) ? sdf.format(new Date(MapTools.isBlank(dataObjectId) ? (System.currentTimeMillis()) : Long.parseLong(dataObjectId.substring(0, 13)))) : inMap.get(readTime));

        String currentResult = inMap.containsKey("dataState") ? "dataState" : (inMap.containsKey("abnormalState") ? "abnormalState" : inMap.containsKey("securityInspectionResults") ? "securityInspectionResults" : null);
        if (MapTools.isNotBlank(readAddess) && Objects.nonNull(inMap.get(currentResult))) {
            inMap.put("currentResult", inMap.get(currentResult));
        }

        if (inMap.containsKey("dataType") && Objects.equals(inMap.get("dataType"), "BSM")) {
            inMap.put("nodeCode", "CHECKIN");
        }
        if (Objects.nonNull(inMap.get("readAddess")) && "ARRIVED,LOAD,INFL,SECURITY,SORT,UNLOAD".contains(inMap.get("readAddess").toString())) {
            inMap.put("nodeCode", inMap.get("readAddess"));
        }
        if (Objects.nonNull(inMap.get("securityInspectionResults"))) {
            inMap.put("nodeCode", "SECURITY");
        }
        Object sourceSign = inMap.get("sourceSign");
        Object secondary_code = inMap.get("secondary_code");
        Object location_describe = inMap.get("location_describe");
        Object to_location_describe = inMap.get("to_location_describe");
        if (Objects.nonNull(sourceSign)) {
            if (!inMap.containsKey("nodeCode") && Objects.equals(sourceSign, "T") && Objects.equals(secondary_code, "G")) {
                inMap.put("nodeCode", "TRANSFER");
            }
            if (!inMap.containsKey("nodeCode") && !Objects.equals(sourceSign, "X") && Objects.equals(location_describe, "RM") && Objects.equals(to_location_describe, "AC")) {
                inMap.put("nodeCode", "INFL");
            }
            if (!inMap.containsKey("nodeCode") && Objects.equals(sourceSign, "X") && Objects.equals(location_describe, "AC")) {
                inMap.put("nodeCode", "UNLOAD");
            }
            if (!inMap.containsKey("nodeCode") && !Objects.equals(sourceSign, "X") && Objects.equals(secondary_code, "S")) {
                inMap.put("nodeCode", "SORT");
            }
            if (!inMap.containsKey("nodeCode") && !Objects.equals(sourceSign, "X") && Objects.equals(secondary_code, "R")) {
                inMap.put("nodeCode", "LOAD");
            }
            if (!inMap.containsKey("nodeCode") && Objects.equals(sourceSign, "X") && !Objects.equals(secondary_code, "S")) {
                inMap.put("nodeCode", "ARRIVED");
            }
        }
        return inMap;
    }

    /**
     * xml转map 定制化
     *
     * @param xml                     字符串
     * @param isTwoDimensionalization 是否需要二维化
     * @param noDiKaList              是否需要dika
     * @param translateName           需要转义的名称
     * @return code message returnData
     */
    public static Map<String, Object> customMadexmlToMap(String xml, boolean isTwoDimensionalization, Object noDiKaList, Map<String, String> translateName) {
        Map<String, Object> returnMap = xmlToMapFormat(xml); //  code ,message ,returenData
        if (returnMap.get("code").equals("-1")) return returnMap;
        if (isTwoDimensionalization) {
            Map<String, Object> xmlMap = (Map<String, Object>) returnMap.get("returnData");
            returnMap = twoDimensionalizationFormat(xmlMap, noDiKaList); //  code ,message ,returenData
            if (returnMap.get("code").equals("-1")) return returnMap;
        }
        if (null != translateName && !translateName.isEmpty()) {
            Object returnData = returnMap.get("returnData"); //  code ,message ,returenData
            if (returnData instanceof Map) {
                List<Map<String, Object>> inData = new ArrayList<>();
                inData.add((Map) returnData);
                returnMap = translateMapName(inData, translateName); // code message retrunData
            } else if (returnData instanceof List) {
                returnMap = translateMapName((List) returnData, translateName); // code message retrunData
            }
        }
        Object rsData = returnMap.get("returnData");
        if (rsData instanceof List<?>) {
            for (Map<String, Object> tempMap : (List<Map<String, Object>>) rsData) {
                tempMap.values().removeAll(Collections.singleton(null));
                tempMap.values().removeAll(Collections.singleton(""));
            }
        }
        if (rsData instanceof Map<?, ?> temMap) {
            temMap.values().removeAll(Collections.singleton(null));
            temMap.values().removeAll(Collections.singleton(""));
        }
        return returnMap;
    }


    public static Map<String, Object> customMadeListxmlToMap(List<String> xmlList, boolean isTwoDimensionalization, Object noDiKaList, Map<String, String> translateName) {
        if (Objects.isNull(xmlList) || xmlList.size() == 0) {
            return MapTools.processSuccess(null);
        }
        if (xmlList.size() == 1) {
            return customMadexmlToMap(xmlList.get(0), isTwoDimensionalization, noDiKaList, translateName);
        }
        List<Map<String, Object>> batchList = new ArrayList<>();
        xmlList.forEach(xml -> {
            Map<String, Object> stringObjectMap = customMadexmlToMap(xml, isTwoDimensionalization, noDiKaList, translateName);
            if ("-1".equals(stringObjectMap.get("code").toString())) {
                stringObjectMap.put("returnData", new ArrayList<>());
            }
            batchList.addAll((List) stringObjectMap.get("returnData"));
        });
        return processSuccess(batchList);
    }


    /**
     * map key值转换
     *
     * @param returnDataList
     * @param translateName
     * @return
     */
    public static Map<String, Object> translateMapName(List<Map<String, Object>> returnDataList, Map<String, String> translateName) {
        try {
            List<Map<String, Object>> dataList = new ArrayList<>();
            for (Map<String, Object> returnData : returnDataList) {
                Map<String, Object> temMap = new HashMap<>();
                for (String key : returnData.keySet()) {
                    if (translateName.containsKey(key)) {
                        temMap.put(translateName.get(key), returnData.get(key));
                    }
                }
                dataList.add(temMap);
            }
            return processSuccess(dataList);
        } catch (Exception e) {
            return processFail("translateMapName替换异常");
        }
    }

    /**
     * xml 转map 通用化
     *
     * @param xml
     * @return
     */
    public static Map<String, Object> xmlToMapFormat(String xml) {
        try {
            String xmlStr = xml.substring(xml.contains("<") ? xml.indexOf("<") : 0, xml.lastIndexOf(">") > 0 ? xml.lastIndexOf(">") + 1 : xml.length());
            xmlStr = (xmlStr.contains("<?xml") ? "" : "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n").concat(xmlStr);
            Document doc = DocumentHelper.parseText(xmlStr);
            Element root = doc.getRootElement();
            Map<String, Object> returnMap = xmlToMap(root);
//            returnMap.put("souceData", xml);
            if ("true".equals(AppConfig.SOURCE_DATA)){
                returnMap.put("sourceData", xml);
            }
            return processSuccess(returnMap);
        } catch (Exception e) {
            Map<String, Object> errMap = processFail("入参不是规则的xml ：" + xml);
//            errMap.put("souceData", xml);
            if ("true".equals(AppConfig.SOURCE_DATA)){
                errMap.put("sourceData", xml);
            }
            return errMap;
        }
    }

    /**
     * xml 转map 通用化
     *
     * @param xmlList
     * @return
     */
    public static Map<String, Object> xmlToMapFormatObj(Object xmlList) {
        try {
            List<Map<String, Object>> rsList = new ArrayList<>();
            if (xmlList instanceof List<?> tpList) {
                for (Object o : tpList) {
                    rsList.add(xmlToMapFormat(Objects.toString(o)));
                }
            } else {
                rsList.add(xmlToMapFormat(Objects.toString(xmlList)));
            }
            return xmlList instanceof List<?> ? MapTools.processSuccess(rsList) : rsList.get(0);
        } catch (Exception e) {
            Map<String, Object> errMap = processFail("入参不是规则的xml");
//            errMap.put("souceData", xmlList);
            if ("true".equals(AppConfig.SOURCE_DATA)){
                errMap.put("sourceData", xmlList);
            }
            return errMap;
        }
    }


    /**
     * map 二维化
     *
     * @param inMap      需要二维化map
     * @param noDiKaList 需要dika
     * @return
     */
    public static Map<String, Object> twoDimensionalizationFormat(Map<String, Object> inMap, Object noDiKaList) {
        try {
            List<Map<String, Object>> returnDataList = new ArrayList<>();
            Map<String, Object> toInMap = MapTools.clone(inMap);
            if (Objects.nonNull(toInMap) && !toInMap.isEmpty()) {
                returnDataList = twoDimensionalization(toInMap, "", noDiKaList); /*生命周期未添加*/
            }
            return processSuccess(returnDataList);
        } catch (Exception e) {
            return processFail(LogUtils.getException(e));
        }
    }

    public Map<String, Object> collectInsertOrUpdate(List<String> ids, String sourceTable, String sourceKey, String targetTable, List<String> uniqueList, boolean isActive, List<String> targetNull, String dbConnect) {
        if ("".equals(dbConnect) || ids.isEmpty() || uniqueList.isEmpty()) {
            return processSuccess(null);

        }
        MyDbHelper myDbHelper = ObjectMap.getordropMyDbHelper(dbConnect);
        /*检测数据库中是否存在对应的表*/
        Map<String, String> hasTableMap = myDbHelper.queryForObject("select COUNT(1) as count from information_schema.TABLES  where TABLE_NAME = ?", targetTable);
        if (hasTableMap.get("code").equals("-1")) {
            return processFail("算法工具类获取目标表结构执行SQL失败： " + hasTableMap.get("message"));
        }
        String count = hasTableMap.get("returnData");
        if ("".equals(count) || "0".equals(count)) {
            if (isActive) {
                try {
                    String tempSql = "CREATE TABLE if not EXISTS " + targetTable + " (" + targetTable + "_id int(11) NOT NULL AUTO_INCREMENT  PRIMARY KEY,`dataObjectId` VARCHAR(32) DEFAULT NULL ,`createtime` timestamp not null default current_timestamp) ";
                    myDbHelper.execute(tempSql);
                } catch (Exception e) {
                    return processFail("collectInsertOrUpdate动态创建表结构失败 " + LogUtils.getException(e));
                }
            } else {
                return processFail("目标表不是动态表，数据库中不存在");
            }
        }

        /*检测数据库中是否存在对应表的唯一键*/
        List<String> allColumn = new ArrayList<>();
        List<String> tempUnique;
        List<String> addIndex = new ArrayList<>();
        tempUnique = MapTools.deepCopy(uniqueList);
        tempUnique = (tempUnique == null) ? new ArrayList<>() : tempUnique;
        Map<String, Object> desc_1Map = myDbHelper.queryByParamsReturnList("desc " + targetTable);
        List<Map<String, Object>> mapList = MapTools.getMapList(desc_1Map);
        if (!desc_1Map.get("code").equals("0") || null == mapList) {
            return processFail("目标表不是动态表，数据库中不存在");
        }
        for (String unique : uniqueList) {
            for (Map<String, Object> fieldMap : mapList) {
                String field = fieldMap.get("Field").toString();
                Object key = fieldMap.get("key");
                allColumn.add(field);
                if (field.equals(unique)) {
                    tempUnique.remove(unique);
                    if (Objects.isNull(key)) {
                        addIndex.add(field);
                    }
                }
            }
        }
        for (String field : tempUnique) {
            myDbHelper.execute("alter table " + targetTable + " add " + field + " text ");
        }
        if (!addIndex.isEmpty()) {
            myDbHelper.execute("alter table " + targetTable + " add  UNIQUE(" + String.join(",", addIndex) + ")");
        }
        tempUnique = MapTools.deepCopy(uniqueList);
        try {
            Map<String, Object> sourceDataListResult = myDbHelper.queryByParamsReturnList("select * from " + sourceTable + " where " + sourceKey + " in (" + String.join(",", ids) + ")");
            List<Map<String, Object>> tempSourceData = MapTools.getMapList(sourceDataListResult);
            if (!sourceDataListResult.get("code").equals("0") || null == tempSourceData) {
                return processFail("查询数据数据失败");
            }
            for (Map<String, Object> currentData : tempSourceData) {
                List<String> currentKeysList = new ArrayList<>(currentData.keySet());
                for (String key : currentKeysList) {
                    if (!allColumn.contains(key)) {
                        myDbHelper.execute("alter table " + targetTable + " add " + key + " text ");
                    }
                }
                List<String> uniqueKey = new ArrayList<>();
                List<Object> uniqueValue = new ArrayList<>();
                List<Object> columnValue = new ArrayList<>();
                List<String> flag = new ArrayList<>();
                for (String tpKey : currentData.keySet()) {
                    Object tpValue = currentData.get(tpKey);
                    if (Objects.isNull(tpValue) && !targetNull.contains(tpKey)) {
                        currentKeysList.remove(tpKey);
                        continue;
                    }
                    if (tempUnique.contains(tpKey)) {
                        uniqueValue.add(tpValue);
                        uniqueKey.add(tpKey);
                        currentKeysList.remove(tpKey);
                        continue;
                    }

                    if (tpKey.equals("createtime") || tpKey.equals(sourceKey)) {
                        currentKeysList.remove(tpKey);
                        continue;
                    }
                    flag.add("?");
                    columnValue.add(tpValue.toString());
                }
                String sql = "select count(1) count from " + targetTable + " where " + String.join(" and ", updateList(uniqueKey, "=?"));
                Map<String, String> hasTargetTableMap = myDbHelper.queryForObject(sql, uniqueValue.toArray());
                if (hasTargetTableMap.get("code").equals("-1")) {
                    return processFail("算法工具类获取目标表结构执行SQL失败： " + hasTargetTableMap.get("message"));
                }
                String targetCount = hasTargetTableMap.get("returnData");
                if ("".equals(targetCount) || "0".equals(targetCount)) {
                    currentKeysList.addAll(uniqueKey);
                    for (String ignored : uniqueKey) {
                        flag.add("?");
                    }
                    String tempSql = "insert into " + targetTable + "(" + String.join(",", currentKeysList) + ")" + "values  (" + String.join(",", flag) + ")";
                    columnValue.addAll(uniqueValue);
                    myDbHelper.updateByCondition(tempSql, null, columnValue.toArray());
                } else {
                    columnValue.addAll(uniqueValue);
                    myDbHelper.updateByCondition("update " + targetTable + " set " + String.join(",", updateList(currentKeysList, "=?")) + " where " + String.join(" and ", updateList(uniqueKey, "=?")), null, columnValue.toArray());
                }
            }
        } catch (Exception e) {
            return processFail("collectInsertOrUpdate SQL 执行异常 " + LogUtils.getException(e));
        }
        return processSuccess(null);
    }

    private static List<Map<String, Object>> twoDimensionalization(Map<String, Object> inMap, String keyName, Object noDiKaList) {
        //初始化一个返回数组List<map<string,object>> returnList
        List<Map<String, Object>> returnList = new ArrayList<>();
        // 默认添加一个空MAP----returnList.add(new map)//目的是确保计算迪卡乘积以及最后的单值赋值不出现问题
        returnList.add(new HashMap<>());
        //循环入参inMap
        for (String currentKeyName : inMap.keySet()) {
//            组装键名itemKeyName = (keyName为空？'':keyName) + '_' + currentKeyName
            String itemKeyName = (Objects.equals(keyName, "") ? "" : keyName + '_') + currentKeyName;//组装键名
            Object currentValue = inMap.get(currentKeyName);
            if (currentValue instanceof List) {
                List<Map<String, Object>> tempList = new ArrayList<>();
                List<Object> list = (List<Object>) currentValue;
                if (Objects.nonNull(noDiKaList) && noDiKaList instanceof List<?> tpList && tpList.contains(itemKeyName)) {
                    Map<String, Object> tempMap = new HashMap<>();
                    tempMap.put(itemKeyName, currentValue.toString());
                    tempList.add(tempMap);
                } else {
                    for (Object o : list) {
                        if (o instanceof Map) {
                            List<Map<String, Object>> tempListMap = twoDimensionalization((Map<String, Object>) o, itemKeyName, noDiKaList);
                            if (noDiKaList instanceof String inStr && inStr.equals("noDika")) {
                                tempList.addAll(tempListMap);
                            } else {
                                tempListMap = dikaMap(returnList, tempListMap);//计算迪卡乘积
                                tempList.addAll(tempListMap);
                            }
                        } else {
                            tempList.add(new HashMap<>() {{
                                put(itemKeyName, o);
                            }});
                        }
                    }
                }
                returnList = tempList;
            } else {
                if (currentValue instanceof Map) {
                    List<Map<String, Object>> tempListMap = twoDimensionalization((Map<String, Object>) currentValue, itemKeyName, noDiKaList);//递归下一级
                    List<Map<String, Object>> tempList = new ArrayList<>();
                    if (Objects.nonNull(noDiKaList) && noDiKaList instanceof List<?> tpList && tpList.contains(itemKeyName)) {
                        Map<String, Object> tempMap = new HashMap<>();
                        tempMap.put(itemKeyName, currentValue.toString());
                        tempList.add(tempMap);
                        returnList = tempList;
                    } else {
                        if (noDiKaList instanceof String inStr && inStr.equals("noDika")) {
                            returnList = tempListMap;
                        } else {
                            returnList = dikaMap(returnList, tempListMap);//计算迪卡乘积后替换当前的returnList
                        }
                    }
                } else {
                    for (Map<String, Object> map : returnList) {//循环returnList
                        map.put(itemKeyName, currentValue);////值不为MAP也不是数组则直接添加
                    }
                }
            }
        }
        return returnList;//返回returnList
    }


    //MAP二维化辅助方法，返回List<map<string,object>>
    private static List<Map<String, Object>> dikaMap(List<Map<String, Object>> parentList, List<Map<String, Object>> childList) {
//        初始化一个返回数组
        List<Map<String, Object>> returnList = new ArrayList<>();
        for (Map<String, Object> childwMap : childList) {
            HashMap<String, Object> childwMapClone = MapTools.clone(childwMap);
            if (Objects.isNull(childwMapClone)) childwMapClone = new HashMap<>();
            for (Map<String, Object> parentMap : parentList) {
                HashMap<String, Object> parentMapClone = MapTools.clone(parentMap);
                if (Objects.isNull(parentMapClone)) {
                    parentMapClone = new HashMap<>();
                }
                parentMapClone.putAll(childwMapClone);
                returnList.add(parentMapClone);//注意使用HashMap.putall实现深拷贝
            }
        }
        return returnList;
    }

    private static Map<String, Object> xmlToMap(Element element) {
        String currentName = element.getName(); //当前节点名称
        String currentValue = MapTools.getText(element.getTextTrim());// 当前节点值

        Map<String, Object> returnMap = new HashMap<>();//初始化一个返回的结构
        Map<String, Object> attrMap = new HashMap<>(); //初始化一个当前节点的属性结构
        List<Attribute> attributes = element.attributes();  //属性处理
        for (Attribute childAttribute : attributes) { //循环当前节点所有属性
            attrMap.put(childAttribute.getName(), childAttribute.getValue());//逐个添加属性值到属性结构
        }
        List<Element> childNodes = element.elements(); //获取当前节点的所有子节点
        if (childNodes.isEmpty() && !attrMap.isEmpty()) { // //无子节点且当前节点存在属性则
            attrMap.put(currentName, currentValue); //当前节点值添加到属性中
        }
        for (Element childElement : childNodes) { //循环所有子节点，如果没有子节点不会进入循环
            String childName = childElement.getName(); //获取子节点名称
            if (attrMap.containsKey(childName)) { //已经存在相同子节点名称的值代表需要形成数组
                Object oldValue = attrMap.get(childName); //获取已经存在的值
                List<Object> newList = new ArrayList<>();
                if (oldValue instanceof List) { //如果旧值已经是数组则
                    newList = (List<Object>) oldValue; //强制转换为数组
                } else {
                    newList.add(oldValue); //转换为数组，旧值添加到数组中
                }
                newList.add(xmlToMap(childElement).get(childName));//添加子节点的数据，递归获取子节点的数据
                attrMap.put(childName, newList);//添加到返回结构中;
            } else {
                attrMap.put(childName, xmlToMap(childElement).get(childName));//递归获取子节点的数据;
            }
        }
        returnMap.put(currentName, attrMap.isEmpty() ? currentValue : attrMap);//添加当前节点的属性以及值（包括了子节点）
        return returnMap;
    }

    private static String TypeBDateChange(String inDate, String inTime) {
        //如果日期不为空
        if (MapTools.isNotBlank(inDate)) {
            String dayStr = inDate.trim().substring(0, 2);
            String monthStr = EMonth.get(inDate.trim().substring(2, 5));
            Calendar currentDate = Calendar.getInstance();
            int yearStr = currentDate.get(Calendar.YEAR);
            //处理跨年问题
            int systemDay = currentDate.get(Calendar.DATE);
            if (currentDate.get(Calendar.MONTH) + 1 == 12 && systemDay > 20 && Integer.parseInt(monthStr) == 1 && Integer.parseInt(dayStr) < 10) {
                yearStr = yearStr + 1;
            }
            if (currentDate.get(Calendar.MONTH) + 1 == 1 && systemDay < 10 && Integer.parseInt(monthStr) == 12 && Integer.parseInt(dayStr) > 20) {
                yearStr = yearStr - 1;
            }
            return yearStr + "-" + monthStr + "-" + dayStr;
        }
        if (MapTools.isNotBlank(inTime)) {
            String dateType = inTime.substring(inTime.length() - 1);
            dateType = MapTools.isNumber(dateType) ? "L" : dateType;
            String h = inTime.length() > 1 ? inTime.substring(0, 2) : "00";
            String m = inTime.length() > 3 ? inTime.substring(2, 4) : "00";
            String s = inTime.length() > 5 ? inTime.substring(4, 6) : "00";
            // 组合时间
            return h + ":" + m + ":" + s + " " + dateType;
        }
        return null;
    }

    /**
     * 修改list内容
     *
     * @param list 源list
     * @param spc  针对每条数据添加的字符
     * @return 新list
     */
    private List<String> updateList(List<String> list, String spc) {
        return list.stream().map(x -> x + spc).collect(Collectors.toList());
    }
}