package org.bfkj.utils;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.stereotype.Component;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class ImportExcelUtils {
    SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");


    // 将表格中的数据添加到List集合中
    public Map<String, Object> upload(String fileName, InputStream is, String userName) {
        String dataObjectId = System.currentTimeMillis() +"";
        try {
            File file = new File(System.getProperty("user.dir") + File.separator + "upload" + File.separator + System.currentTimeMillis() + "_" + fileName);
            String isUpLoad = saveFile(file, is);
            if (null != isUpLoad) {
                return MapTools.processFail(isUpLoad);
            }
            FileInputStream fileInputStream = new FileInputStream(file);
            Workbook workbook = getWorkbook(file.getName(), fileInputStream);
            List<Map<String, Object>> tempMapList = new ArrayList<>();
            int number = Math.min(workbook.getNumberOfSheets(),1); // 如果sheet为空则
            DataFormatter formatter = new DataFormatter();
            for (int i = 0; i < number; i++) {
                Sheet sheet = workbook.getSheetAt(i); // 获取表格页码
                if (sheet != null) {
                    int rowNum = sheet.getLastRowNum(); // 获取该页表共有多少行
                    for (int j = 1; j < rowNum+1; j++) {  // 一般来说第一行是标题,所以第二行开始读取
                        Row row = sheet.getRow(j); // 获取表格行
                        if (Objects.isNull(row)){
                            continue;
                        }
                        int rowNum1 = row.getLastCellNum(); // 第一行
                        HashMap<String, Object> tempMap = new HashMap<>();
                        for (int col = 0; col < rowNum1; col++) {
                            Cell cell = row.getCell(col);
                            tempMap.put("C"+col, cell == null ? null : formatter.formatCellValue(cell));
                        }
                        Date now2 = Calendar.getInstance().getTime();
                        tempMap.put("createtime",format1.format(now2));
                        tempMap.put("dataObjectId",dataObjectId);
                        tempMap.put("userName",userName);
                        if (tempMap.keySet().size() > 2){
                            tempMapList.add(tempMap);
                        }
                    }
                }
            }
            return MapTools.processSuccess(tempMapList);
        } catch (Exception e) {
            return MapTools.processFail("文件上传失败： " + LogUtils.getException(e));
        }
    }
    /**
     * 文件保存到指定的位置
     *
     * @param is
     * @return
     */
    public String saveFile(File file, InputStream is) {
        BufferedInputStream in = null;
        BufferedOutputStream out = null;
        try {
            in = new BufferedInputStream(is);
            out = new BufferedOutputStream(new FileOutputStream(file));
            int len;
            byte[] b = new byte[1024];
            while ((len = in.read(b)) != -1) {
                out.write(b, 0, len);
            }
            return null;
        } catch (Exception e) {
            return "文件保存失败";
        } finally {
            try {
                if (in != null) in.close();
            } catch (IOException e) {
                System.out.println("文件保存 关闭IN失败 ");
            }
            try {
                if (out != null) out.close();
            } catch (IOException e) {
                System.out.println("文件保存 关闭Out失败");
            }
        }
    }

    // 判断传入的文件是哪种类型的excel文件
    public Workbook getWorkbook(String fileName, InputStream is) throws Exception {
        Workbook workbook = null;
        String name = fileName.substring(fileName.lastIndexOf("."));
        System.out.println(name);
        if (".xls".equals(name)) {
            workbook = new HSSFWorkbook(is);
        } else if (".xlsx".equals(name)) {
            workbook = new XSSFWorkbook(is);
        } else {
            throw new Exception(" 请上传.xls/.xlsx格式文件！");
        }
        return workbook;
    }


}
