package org.bfkj.utils;


import org.bfkj.config.AppConfig;
import org.bfkj.config.ElasticsearchConfig;
import org.bfkj.config.SpringContextApplication;
import org.bfkj.protocol.ElasticHandler;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

public class LogUtils { // 依据服务编号ID获取服务类型，如果是异步 异常日志无条件写入到本地数据库（每个服务都有这样的一个数据库） ，在程序启动时会扫描此表，重新进行，如果成功则从此表中移除
    public static Map<String, List<Object>> logErrorMapList = new HashMap<>();
    public static Map<String, List<Object>> logSuccessMapList = new HashMap<>();
    public static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    public static  void log(String location, String success, Object calculationLocation, String logContent, String serviceId,  Object iNDataContent, String dataObjectId, Object OutDataContent, Object event) {
        ThreadPoolTaskExecutor threadPoolTaskExecutor = SpringContextApplication.getBean("ThreadPoolTaskExecutor");
        threadPoolTaskExecutor.submit(() ->
                logWrite(location, success, calculationLocation, logContent, serviceId, iNDataContent, dataObjectId, OutDataContent, event)
        );
    }

    public static void logWrite(String location, String success, Object calculationLocation, String logContent, String serviceId, Object iNDataContent, String dataObjectId, Object OutDataContent, Object event) {
        String inData = MapTools.objToJSONStr(iNDataContent);
        try {
            Map<Object, Object> filterMap = Objects.nonNull(iNDataContent) && iNDataContent instanceof List<?>  tempList && tempList.size() == 2 ? (Map) (((List) iNDataContent).get(1)) : null;
            if (Objects.nonNull(filterMap) && Objects.nonNull(filterMap.get("returnData")) && filterMap.get("returnData").toString().equals("[]")) {
                return;
            }
        } catch (Exception e) {
            System.out.println("过滤：读取空消息： 出现错误".concat(LogUtils.getException(e)).concat(" iNDataContent: ").concat(inData == null ? "" : inData));
            return;
        }
        ElasticsearchConfig elasticsearchConfig = SpringContextApplication.getBean("ElasticBean");
        if (elasticsearchConfig.isOpen()) {
            HashMap<Object, Object> esSendData = new HashMap<>();
            esSendData.put("success", success);
            esSendData.put("location", location);
            esSendData.put("logContent", logContent);
            esSendData.put("ctime", new Date());
            esSendData.put("serviceId", serviceId);
            esSendData.put("workId", AppConfig.WORK_ID);
            esSendData.put("event", Objects.isNull(event) ? null : event.toString());
            esSendData.put("iNDataContent", inData);
            esSendData.put("outDataContent", MapTools.objToJSONStr(OutDataContent));
            esSendData.put("calculationLocation", Objects.isNull(calculationLocation) ? null : calculationLocation.toString());
            esSendData.put("dataObjectId", dataObjectId);
            ElasticHandler elasticHandler = new ElasticHandler();
            String connectConfig = MapTools.objToJSONStr(elasticsearchConfig);
            boolean b;
            if (success.equals("0")) {
                b = elasticHandler.sendData(elasticsearchConfig.getSuccessIndexName(), new ArrayList<>() {{
                    add(esSendData);
                }}, connectConfig, serviceId);
            } else {
                b = elasticHandler.sendData(elasticsearchConfig.getErrorIndexName(), new ArrayList<>() {{
                    add(esSendData);
                }}, connectConfig, serviceId);
            }
            if (b) {
                return;
            }
        }
        List<Object> logParam = new ArrayList<>() {{
            add(success);
            add(location);
            add(logContent);
            add(sdf.format(new Date()));
            add(serviceId);
            add(AppConfig.WORK_ID);
            add(Objects.isNull(event) ? null : event.toString());
            add(inData);
            add(MapTools.objToJSONStr(OutDataContent));
            add(Objects.isNull(calculationLocation) ? null : calculationLocation.toString());
            add(dataObjectId);
        }};
        String key = (System.currentTimeMillis() / 1000) + "";
        if (!"0".equals(success)) {
            List<Object> tempLogList = logErrorMapList.containsKey(key) ? logErrorMapList.get(key) : new ArrayList<>();
            tempLogList.add(logParam);
            logErrorMapList.put(key, tempLogList);
        } else {
            List<Object> tempLogList = logSuccessMapList.containsKey(key) ? logSuccessMapList.get(key) : new ArrayList<>();
            tempLogList.add(logParam);
            logSuccessMapList.put(key, tempLogList);
        }
    }

    /**
     * 获取当前行
     *
     * @return
     */
    public static String getLine() {
        StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
        return stackTrace[2].getClassName() + ":" + stackTrace[2].getLineNumber();
    }
    /**
     * 异常位置
     *
     * @param e 异常信息
     * @return
     */
    public static String getException(Exception e) {
        StackTraceElement[] stackTraceElements = Arrays.copyOf(e.getStackTrace(), e.getStackTrace().length < 6 ? e.getStackTrace().length : 5);
        return "异常消息：" + e.getMessage() + "  异常位置：" + Arrays.stream(stackTraceElements).map(it -> String.format("%s:%s:%d", it.getClassName(), it.getMethodName(), it.getLineNumber())).collect(Collectors.joining("\n"));//异常位置
    }

}
