package org.bfkj.utils;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import org.bfkj.config.AppConfig;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.util.StringUtils;

import java.sql.*;
import java.util.Date;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MyDbHelper {//目前只差主键的条件优化
    private HikariDataSource theDataSource;//数据库连接池
    private JdbcTemplate theJdbcTemplate;//JDBC模板

    private List<String> tableList = new ArrayList<>();//当前连接对应数据库的所有表
    private Map<String, List<String>> tableColumn = new HashMap<>();//缓存对象调用过的表的字段列表
    private Map<String, List<String>> tablePrimaryKey = new HashMap<>();//缓存对象调用过的表的主键列表

    Pattern regExpression = Pattern.compile("(?<=《)([^》]+)?(?=》)");//提取书名号变量的正则表达式

    Map<String, String> sqlAuthMap = new HashMap<>();//已经设置好列权限的SQL语句
    private Map<String, String> sqlAllColumn = new HashMap<>();//缓存对象调用过的SQL的字段列表
    Map<String, List<String>> sqlStrVarList = new HashMap<>();//SQL语句的书名号变量列表
    Map<String, String> sqlStrNewSQL = new HashMap<>();//SQL语句更换书名号变量后的可执行SQL


    private boolean isActive = false;
    //还差一个基本字段列表，避免动态表自动生成(event,page,pagesize,serviceid)
    private String errorMessage = null; //当前数据输入处理对象不可用信息

    public MyDbHelper(String connectStr) { // 需要json字符串
        Map<String, Object> connectMaps = null;
        try {
            connectMaps = (Map<String, Object>) MapTools.strToObj(connectStr);
        } catch (Exception e) {
            errorMessage = "数据库连接字符串非法: ".concat(connectStr);
            return;
        }
        if (connectMaps == null || Objects.isNull(connectMaps.get("url")) || Objects.isNull(connectMaps.get("driver-class-name"))) {
            errorMessage = "数据库连接字符串非法: ".concat(connectStr);
            return;
        }
        HikariConfig config = new HikariConfig();
        config.setMaximumPoolSize(Math.min(Runtime.getRuntime().availableProcessors(), 8));//最大核心数:当前可用CPU数，最大16
        config.setKeepaliveTime(60000); //用于跟数据库保持心跳连接
        config.setMaxLifetime(60000 * 60 * 4); //4小时 接在池中的最大生存时间，超过该时间强制逐出
        config.addDataSourceProperty("cachePrepStmts", "true"); //是否自定义配置，为true时下面两个参数才生效
        config.addDataSourceProperty("prepStmtCacheSize", "250"); //连接池大小默认25，官方推荐250-500
        config.addDataSourceProperty("prepStmtCacheSqlLimit", "2048"); //单条语句最大长度默认256，官方推荐2048
        config.addDataSourceProperty("useServerPrepStmts", "true"); //新版本MySQL支持服务器端准备，开启能够得到显著性能提升
        config.addDataSourceProperty("useLocalSessionState", "true");
        config.addDataSourceProperty("useLocalTransactionState", "true");
        config.addDataSourceProperty("rewriteBatchedStatements", "true");//批量新增
        config.addDataSourceProperty("cacheResultSetMetadata", "true");
        config.addDataSourceProperty("cacheServerConfiguration", "true");
        config.addDataSourceProperty("elideSetAutoCommits", "true");
        config.addDataSourceProperty("maintainTimeStats", "false");
//        config.setConnectionTimeout();
        if (Objects.nonNull(connectMaps.get("username"))) {
            config.setUsername(connectMaps.get("username").toString());
        }
        if (Objects.nonNull(connectMaps.get("password"))) {
            config.setPassword(connectMaps.get("password").toString());
        }
        config.setJdbcUrl(connectMaps.get("url").toString());
        config.setDriverClassName(connectMaps.get("driver-class-name").toString());

        config.setPoolName("");

        theDataSource = new HikariDataSource(config);
        theJdbcTemplate = new JdbcTemplate(theDataSource);
        try (Connection theConnect = theDataSource.getConnection();//用于批量新增以及获取当前数据库的相关信息
             ResultSet tablesRsesult = theConnect.getMetaData().getTables(theConnect.getCatalog(), theConnect.getCatalog(), null, new String[]{"TABLE"});) {
            while (tablesRsesult.next()) {//缓存当前数据库的所有表名
                tableList.add(tablesRsesult.getString("TABLE_NAME"));//获取表名
            }
        } catch (Exception e) {
            errorMessage = LogUtils.getException(e);
        }
    }

    //统一数据处理，兼容增删改查
    public Map<String, Object> generalProcess(Map<String, Object> calcInfo, Object inParams, String dataObjectId, Map<String, Object> inputData) {
        try {
            isActive = true;
            String tableName = (Objects.isNull(calcInfo) || Objects.isNull(calcInfo.get("sourceObjectName"))) ? null : calcInfo.get("sourceObjectName").toString();//获取表名
            String sqlStr = (Objects.isNull(calcInfo) || Objects.isNull(calcInfo.get("computing_expression"))) ? null : calcInfo.get("computing_expression").toString();//获取SQL语句
            String event = MapTools.isNotBlank(sqlStr) ? AppConfig.staticEvent.get(sqlStr.toLowerCase().trim().substring(0, 6))//SQL语句方式，事件由SQL确定;
                    : (Objects.nonNull(calcInfo) && Objects.nonNull(calcInfo.get("event"))) ? calcInfo.get("event").toString() //优先算法定义的事件
                    : (Objects.nonNull(inputData) && Objects.nonNull(inputData.get("event"))) ? inputData.get("event").toString() : null;//否则上传参数的事件
            //-----入参检测

            if (Objects.isNull(event) && MapTools.isNotBlank(sqlStr) && sqlStr.toLowerCase().contains(") union")) {
                event = Objects.toString(calcInfo.get("event"));
            }

            if (MapTools.isBlank(event) || (MapTools.isBlank(tableName) && MapTools.isBlank(sqlStr))) {//如果表名为空 且SQL为空则返回错误
                return processFail("generalProcess函数： 表为空且SQL为空或事件为空", null, inParams);
            }
            Map<String, Map<String, Object>> execSQLMap = new HashMap<>();//最终需要执行的SQL语句以及对应的执行参数集合{sqlStr:[[参数]]}
            Map<String, Map<String, Object>> inertSQLMap = new HashMap<>();
            List<String> authColumn = (Objects.nonNull(inputData) && Objects.nonNull(inputData.get("authColumn"))) ? (List<String>) inputData.get("authColumn") : new ArrayList<>();
            List<Map<String, Object>> rowAuth = (Objects.nonNull(inputData) && Objects.nonNull(inputData.get("rowAuth"))) ? (List<Map<String, Object>>) inputData.get("rowAuth") : new ArrayList<>();
            String newSqlStr = sqlStr;//默认新的SQL执行语句等于算法设定
            String allSQLColumn = "";//
            List<String> varList = new ArrayList<>();//
            String rowCountSql = "";
            if (MapTools.isNotBlank(sqlStr)) {//SQL语句方式：1、书名号；2、Where；3、SQL语句批量
                sqlStr = standarSQLStr(sqlStr); // todo sqlStr??  // select * from xx
                newSqlStr = setAuthColumn(newSqlStr, authColumn, event);//列权限设置：1、获取所有列用于where的组建；2、返回修订后的SQL语句（去掉权限外的列）//sql ,allSQLCOlumn
                allSQLColumn = sqlAllColumn.get(newSqlStr.concat(authColumn.toString()));//用于where组建的字段：1、Select优先依据伪列名；2、其它依据表列名
                varList = getSQLVarList(newSqlStr);//获取SQL语句中的书名号变量列表，同时也将书名号进行了替换《中内容》 select * from xx
                newSqlStr = sqlStrNewSQL.get(newSqlStr);//获取已经将书名号替换为?后的SQL语句
            } else {
                Map<String, Object> metaDataMap = getMetaDataByTable(tableName, Objects.equals("true", calcInfo.get("isActiveTable")));//获取表的所有字段列表
                if ("-1".equals(metaDataMap.get("code"))) {//获取字段失败，大概率是动态表创建失败
                    return metaDataMap;
                }
                allSQLColumn = String.join(",", tableColumn.get(tableName));//依据表名获取所有列
            }
            List<Map<String, Object>> initParam = new ArrayList<>();
            if (Objects.nonNull(inParams)) {
                if (inParams instanceof List tempList) {
                    if (tempList.size() == 0) {
                        initParam.add(new HashMap<>());
                    } else {
                        initParam = (List<Map<String, Object>>) inParams;
                    }
                }
                if (inParams instanceof Map<?, ?>) {
                    initParam.add((Map<String, Object>) inParams);
                }
            } else {
                initParam.add(new HashMap<>());
            }
            int whereIndex = 0;
            for (Map<String, Object> signParam : initParam) {//循环入口参数，可以开启多线程执行
                String execSQLStr = newSqlStr;
                Map<String, Object> dbFilter = getdbFilter(varList, event, signParam, rowAuth, allSQLColumn);//获取条件，1、返回where字符串；2、返回执行参数 where  值
                List<Object> filterList = (List<Object>) dbFilter.get("dbFilter");//执行参数---
                String whereStr = (String) dbFilter.get("whereStr");//where字符串--依据参数变化不可缓存

                List<Object> dbValueList = new ArrayList<>();
                String newEvent = "6".equals(event) && MapTools.isNotBlank(tableName) ? isExist(tableName, whereStr, filterList) : event;//如果是事件6则需要先进行判断后再决定事件
                Map<String, Object> dbValueInfo = MapTools.isNotBlank(sqlStr) ? getdbValueForSql(execSQLStr, varList, signParam) : getdbValueForTable(tableName, newEvent, authColumn, signParam, Objects.equals("true", calcInfo.get("isActiveTable")));//获取执行的参数值集合，SQL语句方式针对的是书名号变量，注意使用未替换书名号的SQL语句
                dbValueList = (List<Object>) dbValueInfo.get("dbValue");//执行参数
                execSQLStr = dbValueInfo.get("newSqlStr").toString();//修订后的SQL语句，针对in方式进行修订 select * from xx
                rowCountSql = "0".equals(newEvent) && MapTools.isNotBlank(allSQLColumn) ? execSQLStr.replace(allSQLColumn, " count(1) ") : "";//查询则创建rowCountSql
                if (MapTools.isNotBlank(whereStr) && !"1".equals(newEvent)) {//如果where字符串不为空
                    execSQLStr = execSQLStr.concat(execSQLStr.lastIndexOf(")") >= execSQLStr.toLowerCase().lastIndexOf(" where ") ? " where " : " and ").concat("(").concat(whereStr).concat(")");
                    rowCountSql = MapTools.isBlank(rowCountSql) ? "" : rowCountSql.concat(rowCountSql.lastIndexOf(")") >= rowCountSql.lastIndexOf(" where ") ? " where " : " and ").concat("(").concat(whereStr).concat(")");
                    dbValueList.addAll(filterList);//组合执行参数
                }

                Map<String, Object> sqlParmMap = new HashMap<>();
                sqlParmMap.put("rowCountSql", rowCountSql);//行总数查询
                sqlParmMap.put("event", newEvent);//事件
                Map<String, Map<String, Object>> tempSQLMap = "1".equals(newEvent) ? inertSQLMap : execSQLMap;

                List signParamList = Objects.nonNull(tempSQLMap.get(execSQLStr)) && Objects.nonNull(tempSQLMap.get(execSQLStr).get("signParam")) ? (List) tempSQLMap.get(execSQLStr).get("signParam") : new ArrayList<>();//获取对应SQL语句的执行参数，没有则默认为空数组
                List dbParam = Objects.nonNull(tempSQLMap.get(execSQLStr)) && Objects.nonNull(tempSQLMap.get(execSQLStr).get("dbParam")) ? (List) tempSQLMap.get(execSQLStr).get("dbParam") : new ArrayList<>();//获取对应SQL语句的执行参数，没有则默认为空数组
                signParamList.add(signParam);
                sqlParmMap.put("signParam", signParamList);//上传的参数
                if ("1".equals(newEvent)) {
                    //参数一致时，去重执行
                    dbParam.add(dbValueList);//添加当前行的执行参数
                    sqlParmMap.put("dbParam", dbParam);//执行参数
                    inertSQLMap.put(execSQLStr, sqlParmMap);//添加到最终执行列表中
                } else {
                    if (!dbParam.contains(dbValueList)) {//参数一致时，去重执行
                        String batchSQL = Objects.nonNull(execSQLMap.get(execSQLStr)) && Objects.nonNull(execSQLMap.get(execSQLStr).get("batchSQL")) ? execSQLMap.get(execSQLStr).get("batchSQL").toString() : "";
                        if ("0".equals(newEvent) && dbParam.size() > 0) {
                            batchSQL = (MapTools.isBlank(batchSQL) ? execSQLStr : batchSQL).concat(" or (").concat(whereStr).concat(")");
                            List tempList = dbParam.isEmpty() ? new ArrayList() : (List) dbParam.get(0);
                            tempList.addAll(dbValueList);
                            dbParam.clear();
                            dbParam.add(tempList);
                            sqlParmMap.put("batchSQL", batchSQL);
                        } else {
                            dbParam.add(dbValueList);//添加当前行的执行参数
                        }
                        if (execSQLStr.indexOf("{") > 0 && execSQLStr.indexOf("}") > 0) {
                            whereIndex++;
                        }
                    }
                    sqlParmMap.put("dbParam", dbParam);//执行参数
                    if (dbParam.size() == 0 && MapTools.isBlank(whereStr) && "2,3".contains(newEvent)) {
                        sqlParmMap.put("noExec", " 禁止全表更新、删除");//执行参数
                    }
                    execSQLMap.put(execSQLStr, sqlParmMap);//添加到最终执行列表中
                }
            }
            //循环执行execSQLMap
            Map<String, Map<String, Object>> tmpExec = new HashMap<>();
            tmpExec.putAll(inertSQLMap);
            tmpExec.putAll(execSQLMap);
            for (int loop = 0; loop < 2; loop++) {
                Map<String, Map<String, Object>> exeMap = new HashMap<>();
                if (loop == 0) {
                    exeMap = inertSQLMap;
                } else {
                    exeMap = execSQLMap;
                }
                for (String signSql : exeMap.keySet()) {//循环当前MAP，查询必定是单条SQL，SQL语句方式：都是单条，表名方式：查询、新增是单条，更新、删除可能多条，事件6可能多条
                    Map<String, Object> signParamMap = exeMap.get(signSql);
                    String newSignSQL = signSql;
                    if (signSql.indexOf("{") > 0 && signSql.indexOf("}") > 0) {
                        String tempWhere = signSql.substring(signSql.indexOf("{") + 1, signSql.indexOf("}"));
                        String newWhere = tempWhere;
                        for (int i = 0; i < whereIndex - 1; i++) {
                            newWhere = newWhere.concat(" or ").concat(tempWhere);
                        }
                        newWhere = "(".concat(newWhere).concat(")");
                        signParamMap.put("batchSQL", null);
                        signParamMap.put("rowCountSql", null);
                        signSql = signSql.replace(tempWhere, newWhere).replace("{", "").replace("}", "");
                    }

                    Map<String, Object> execSignResult = new HashMap<>();
                    if (signParamMap.containsKey("noExec")) {
                        execSignResult.put("code", "0");
                        execSignResult.put("message", " 禁止全表更新、删除");
                    } else {
                        execSignResult = execSign(sqlStr, signSql, signParamMap, inputData);
                    }
                    if ("-1".equals(execSignResult.get("code"))) {
                        return execSignResult;
                    }
                    signParamMap.put("returnData", execSignResult.get("returnData"));
                    signParamMap.put("rowcount", execSignResult.get("rowcount"));
                    signParamMap.put("sql", signSql);
                    tmpExec.put(newSignSQL, signParamMap);
                }
            }
            Object obj = new Object();
            if (tmpExec.keySet().size() == 1) {
                for (String exc : tmpExec.keySet()) {
                    obj = tmpExec.get(exc);
                    break;
                }
            } else {
                obj = tmpExec;
            }
            return processSuccess(obj);
        } catch (Exception e) {
            return processFail("generalProcess函数：执行出错", LogUtils.getException(e), inParams);
        } finally {
            isActive = false;
        }
    }

    /*标准化SQL*/
    private String standarSQLStr(String sqlStr) {
        String newSqlStr = sqlStr;
        String[] sqlKeyWordList = "select,from,where,group by,limit,order by".split(",", -1);
        for (String sqlKeyWord : sqlKeyWordList) {//循环进行替换
            Pattern p = Pattern.compile(sqlKeyWord, Pattern.CASE_INSENSITIVE);
            Matcher m = p.matcher(newSqlStr);
            newSqlStr = m.replaceAll(sqlKeyWord);
//            newSqlStr = newSqlStr.replaceAll("\\s?(?!)".concat(sqlKeyWord).concat("\\s?"), " ".concat(sqlKeyWord).concat(" "));//关键词小写
        }
        String regx = "\\s+|\t|\r|\n";
        Pattern patt = Pattern.compile(regx);
        Matcher m = patt.matcher(newSqlStr);
        newSqlStr = m.replaceAll(" ");
        newSqlStr = newSqlStr.trim().replaceAll("\\s?[,]\\s?", ",");//去多余空格，去逗号前后空格,全小写
        String lastChar = newSqlStr.substring(newSqlStr.length() - 1);
        if (lastChar.equals(";")) {//最后一个字符不是数字也不是字母，用于去除多余的分号
            newSqlStr = newSqlStr.substring(0, newSqlStr.length() - 1);
        }
        return newSqlStr;
    }

    public Map<String, Object> getAllColumnForSet(String sql) {
        if (sql.lastIndexOf(")") >= sql.lastIndexOf("limit")) {
            sql = sql + " limit 1";
        }
        try (Connection connection = theDataSource.getConnection();
             PreparedStatement ps = connection.prepareStatement(sql);
             ResultSet rs = ps.executeQuery()) {
            ResultSetMetaData rsd = rs.getMetaData();
            List<Map<String, Object>> allColumnList = new ArrayList<>();
            for (int i = 0; i < rsd.getColumnCount(); i++) {
                HashMap<String, Object> tempMap = new HashMap<>();
                String columnName = rsd.getColumnName(i + 1);
                String columnLabel = rsd.getColumnLabel(i + 1);
                tempMap.put("columnName", Objects.isNull(columnLabel) ? columnName : columnLabel);
                tempMap.put("columnType", rsd.getColumnType(i + 1));
                tempMap.put("columnTypeName", rsd.getColumnTypeName(i + 1));
                tempMap.put("columnLable", Objects.isNull(columnLabel) ? columnName : columnLabel);
                allColumnList.add(tempMap);
            }
            return processSuccess(allColumnList);
        } catch (SQLException e) {
            return processFail("getAllColumnForSet 异常： " + LogUtils.getException(e), sql, null);
        }
    }

    public Map<String, Object> getAllColumnTableSet(String tabeName) {
        try (Connection connection = theDataSource.getConnection()) {
            DatabaseMetaData dbMetaData = connection.getMetaData();
            ResultSet rs = dbMetaData.getColumns(connection.getCatalog(), connection.getCatalog(), tabeName, "%");
            List<Map<String, Object>> columnList = new ArrayList<>();
            while (rs.next()) {
                Map<String, Object> tempMap = new HashMap<>();
                tempMap.put("columnName", rs.getString("COLUMN_NAME"));
                tempMap.put("columnType", rs.getString("TYPE_NAME"));
                tempMap.put("columnLable", rs.getString("REMARKS"));
                tempMap.put("columnDescribe", rs.getString("REMARKS"));
                tempMap.put("COLUMN_SIZE", rs.getString("COLUMN_SIZE"));
                columnList.add(tempMap);
            }
            return processSuccess(columnList);
        } catch (Exception e) {
            return processFail("列设置初始化，获取失败".concat(LogUtils.getException(e)), tabeName, null);
        }
    }


    //SQL语句方式设置列权限
    private String setAuthColumn(String sqlStr, List<String> authColumn, String eventStr) {
        try {
            String newSqlStr = sqlStr;//标准化SQL语句，1、去多余空格；2、确保,号不会有空格；3、去最后一位多余的分号
            String sqlAuthKey = newSqlStr.concat(authColumn.toString());//相同的SQL语句相同的列权限，无需每次进行设置，缓存即可
            if (!sqlAuthMap.containsKey(sqlAuthKey)) {//未缓存则重新获取
                List<String> tempColumn = new ArrayList<>();
                String allSQLColumn = getAllColumnBySql(newSqlStr, eventStr); //依据SQL语句获取所有列名，select获取当前语句内的完整字段（包括伪列名），其它获取表结构的完整字段
                if (authColumn.size() > 0 && !"3".equals(eventStr) && MapTools.isNotBlank(allSQLColumn)) {//存在列权限且不是删除且获取到了完整字段列表
                    for (String auColumn : authColumn) { //循环完整字段列表
                        if (allSQLColumn.contains(auColumn)) { //列权限不包含当前字段
                            tempColumn.add(auColumn);
                        }
                    }
                    //todo 涉及到列的取值 存在问题
                    newSqlStr = newSqlStr.replaceFirst(allSQLColumn, " ".concat(String.join(",", tempColumn)));//直接进行清除
                    newSqlStr = newSqlStr.replaceAll("[,]+", ",").replaceAll(", from ", " from ");//清除多余的逗号
                }
                sqlAllColumn.put(sqlAuthKey, allSQLColumn);//缓存当前SQL可用的字段
                sqlAuthMap.put(sqlAuthKey, newSqlStr);//缓存进行过列权限修订的SQL语句
            }
            return sqlAuthMap.get(sqlAuthKey);//返回进行过列权限修订的SQL语句 //
        } catch (Exception e) {
            System.out.println("SQL语句方式设置列权限失败： ".concat(LogUtils.getException(e)));
            return null;
        }
    }

    //依据SQL语句获取所有列名，select获取当前语句内的完整字段（包括伪列名），其它获取表结构的完整字段
    public String getAllColumnBySql(String sqlStr, String event) {
        try {
            if ("0".equals(event)) { //查询：查询不能使用表结构字段，存在伪列名子查询等复杂情况
                int beginIndex = 6;//默认从第一个select后开始进行查找
                String tempSQLStr = sqlStr.toLowerCase();//初始化一个中间变量，不影响原SQL语句，因为中间会把子查询的select替换掉
                String allColumnStr = "";//初始化一个完整字段列表字符串
                String fromStr = " from ";
                while (true) {//循环查找select后的from     //  select (select a as b from x) as t  from tx  id = select * from tx2
                    if (!tempSQLStr.contains(fromStr)) {
                        fromStr = "from";
                    }
                    beginIndex = tempSQLStr.indexOf(fromStr, beginIndex); //从上一次找到的from后面开始查找下一个from，beginIndex的目的是为避免重复查找一个
                    allColumnStr = tempSQLStr.substring(6, beginIndex); //取出查找到的内容
                    int findOne = allColumnStr.indexOf(" select "); //判断是否存在子查询
                    int findTwo = allColumnStr.indexOf("(select "); //判断是否存在子查询
                    if (findOne < 0 && findTwo < 0) {//不存在代表找到的from就是当前第一层查询对应的from
                        break;//找到就跳出循环
                    }
                    tempSQLStr = tempSQLStr.replace(findOne > -1 ? " select " : "(select ", "88888888");//替换掉子查询的select，避免下一次又找到，同时注意替换长度要保持一致，否则影响后面的取值
                    beginIndex = beginIndex + 6;//在当前找到的from位置上加6，下一次查找就不会再找当前的from
                }
                allColumnStr = beginIndex > 6 ? sqlStr.substring(6, beginIndex) : ""; //从原SQL中提取完整的字段，tempSQLStr中子查询select被替换掉了
                return allColumnStr; //使用逗号分割为字段列表
            } else {//增删改只涉及单表，不存在伪列的情况
                String tableName = getTableBySql(sqlStr);//获取表名
                if (MapTools.isBlank(tableName)) {
                    return "";
                }
                getMetaDataByTable(tableName, false);//通过表名获取字段，SQL语句方式不处理动态表
                return String.join(",", tableColumn.get(tableName)); //从缓存中获取
            }
        } catch (Exception e) {
            System.out.println("getAllColumnBySql 异常： ".concat(LogUtils.getException(e)));
            return null;
        }

    }

    //根据sql语句获取表名：查询存在多层多表联合等复杂情况，暂时不支持
    private String getTableBySql(String sqlStr) {
        String[] sqlSplit = sqlStr.split("\\s+", -1);//使用空格进行分组
        return sqlSplit.length < 3 ? null : sqlSplit[0].equals("insert") ? (sqlSplit[2].indexOf("(") > 0 ? sqlSplit[2].substring(0, sqlSplit[2].indexOf("(")) : sqlSplit[2])//insert into 表名(字段名)
                : sqlSplit[0].equals("update") ? sqlSplit[1]//update 表名
                : sqlSplit[0].equals("delete") ? sqlSplit[2] : null;//delete from 表名
    }

    //依据表名获取所有字段
    public Map<String, Object> getMetaDataByTable(String tableName, boolean isActive) {
        tableName = MapTools.isBlank(tableName) ? "(null)" : tableName;
        if (!(tableColumn.containsKey(tableName)) && !"(null)".equals(tableName)) {//对应表的字段未缓存（且表名不为空）则重新获取
            if (isActive && !tableList.contains(tableName)) {//处理动态表：动态表且未包含在当前库
                try {
                    theJdbcTemplate.execute("CREATE TABLE if not EXISTS ".concat(tableName).concat("(").concat(tableName).concat("_id int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,dataObjectId VARCHAR(32),createtime datetime)"));
                    tableList.add(tableName);//添加到表名缓存中
                } catch (Exception e) {
                    return processFail("动态创建表时异常:".concat(LogUtils.getException(e)), tableName, null);
                }
            }
            if (!tableList.contains(tableName)) {//处理完动态表仍旧不存在当前数据库
                return processFail("表名为空或者当前数据库中不存在此表", tableName, null);
            }
            try (Connection connection = theDataSource.getConnection();//获取连接
                 ResultSet columnRsesult = connection.getMetaData().getColumns(connection.getCatalog(), connection.getCatalog(), tableName, null);//获取对应表的字段
                 ResultSet primaryKeyResult = connection.getMetaData().getPrimaryKeys(connection.getCatalog(), connection.getCatalog(), tableName)//获取对应表的主键字段
            ) {
                List<String> columnList = new ArrayList<>();
                while (columnRsesult.next()) {
                    columnList.add(columnRsesult.getString("COLUMN_NAME"));//循环添加字段到缓存
                }
                tableColumn.put(tableName, columnList);//添加到对应表的缓存中
                List<String> primaryKeyList = new ArrayList<>();
                while (primaryKeyResult.next()) {
                    primaryKeyList.add(primaryKeyResult.getString("COLUMN_NAME"));
                }
                tablePrimaryKey.put(tableName, primaryKeyList);//添加到对应表的缓存中
            } catch (Exception e) {
                return processFail("依据表名获取字段信息出错：".concat(LogUtils.getException(e)), tableName, null);
            }
        }
        return processSuccess(null);
    }

    //获取SQL语句中的所有书名号变量
    private List<String> getSQLVarList(String sqlStr) {
        String newSqlStr = sqlStr;//不破坏之前SQL语句
        List<String> sqlvarList = sqlStrVarList.containsKey(sqlStr) ? sqlStrVarList.get(sqlStr) : new ArrayList<>();//优先从缓存中取
        if (!sqlStrNewSQL.containsKey(sqlStr)) {//缓存不存在则重新获取
            Matcher matcher = regExpression.matcher(sqlStr);//正则提取
            while (matcher.find()) {//循环提取到的所有书名号变量
                String varName = matcher.group();
                sqlvarList.add(varName.trim());//添加到变量列表---如果是in则书名号应该设置为《:col》，此处替换不使用？，在变量获取时依据变量进行多个？号替换select* from 行李日志表 where 行李牌号 in (《:行李牌号》)
                newSqlStr = newSqlStr.replaceFirst("《".concat(varName).concat("》"), varName.startsWith(":") ? varName.concat("?") : "?"); //修订SQL
            }
            sqlStrVarList.put(sqlStr, sqlvarList);//添加到缓存
            sqlStrNewSQL.put(sqlStr, newSqlStr);//添加到缓存
        }
        return sqlvarList;
    }

    //获取SQL执行的参数集：针对书名号变量
    private Map<String, Object> getdbValueForSql(String sqlStr, List<String> varList, Map<String, Object> signParam) {
        String newSqlStr = sqlStr;
        List<Object> dbValue = new ArrayList<>();//执行参数
        try {
            Map<String, Object> valueObj = signParam.containsKey("Value") ? (Map<String, Object>) signParam.get("Value") : signParam;//默认取value，否则兼容上层--不记得前端传值是Value还是value
            for (String columnName : varList) {//循环书名号变量名
                String newColumnName = columnName;
                if (newColumnName.startsWith(":")) {//处理in,只是书名号方式存在
                    newColumnName = newColumnName.substring(1);//去掉:才可以取值
                }
                Object currentVar = valueObj.get(newColumnName);//优先从当前MAP中直接获取

                if (columnName.startsWith(":")) {//处理in,只是书名号方式存在
                    String[] varSplit = Objects.isNull(currentVar) ? null : currentVar.toString().split(",", -1);//使用逗号进行分组
                    if (Objects.isNull(varSplit)) {//
                        newSqlStr = newSqlStr.replaceFirst(columnName.concat("?"), "?"); //修订SQL
                    } else {
                        List<String> tempVarList = new ArrayList<>();//准备对应的?
                        for (String signVar : varSplit) {//循环进行替换
                            dbValue.add(signVar);//添加变量
                            tempVarList.add("?");
                        }
                        newSqlStr = newSqlStr.replaceFirst(columnName.concat("?"), String.join(",", tempVarList)); //修订SQL
                    }
                } else {
                    dbValue.add(currentVar);//添加变量
                }
            }
        } catch (Exception e) {
            System.out.println("获取SQL执行的参数集：针对书名号变量:".concat(LogUtils.getException(e)));
        }
        Map<String, Object> returnData = new HashMap<>();
        returnData.put("dbValue", dbValue);//执行参数
        returnData.put("newSqlStr", newSqlStr);//修订后的SQL语句，针对in方式进行修订
        return returnData;

    }

    //组建SQL的where以及对应的执行参数，与SQL语句无关，所以兼容SQL语句方式和表名方式(event,  signParam, rowAuth,allSQLColumn)
    private Map<String, Object> getdbFilter(List<String> varList, String event, Map<String, Object> signParam, List<Map<String, Object>> rowAuth, String allSQLColumn) {
        List<Object> dbFilter = new ArrayList<>();//初始化条件执行参数
        String whereStr = "";//初始化条件字符串
        try {
            if (!"1".equals(event)) {//新增不存在条件，除非insert select才存在

                List<Map<String, Object>> filterList = standarInFilter(signParam, event, rowAuth);//先进行上传条件参数的标准化，同时把行权限进行合并
                if (filterList.size() > 0) {//如果存在上传的条件参数或者存在行权限
                    filterList.get(filterList.size() - 1).put("connector", "lastConnector");//去掉最后一个连接符，减少whereStr字符串的处理
                    for (Map<String, Object> filterMap : filterList) {//循环条件参数集合
                        Object column = filterMap.get("column");//获取列名
//                    && && (MapTools.isBlank(allSQLColumn) || allSQLColumn.contains(column.toString()
                        if (Objects.nonNull(column) && !varList.contains(column.toString())) {//列名不为空 且 (可用列未获取 或者 当前列存在于可用列)
                            String connector = Objects.isNull(filterMap.get("connector")) ? "" : filterMap.get("connector").toString();//处理连接字符串
                            Object comparator = Objects.isNull(filterMap.get("comparator")) ? "" : filterMap.get("comparator").toString();//比较符
                            whereStr = whereStr.concat(filterMap.get("left").toString()).
                                    concat(column.toString()).
                                    concat(comparator.toString()).
                                    concat(" is null ".equals(comparator) ? "" : "?").
                                    concat(filterMap.get("right").toString()).
                                    concat((MapTools.isBlank(connector)) ? " and " : ("lastConnector".equals(connector) ? " " : " ".
                                            concat(connector).concat(" ")));
                            if (!" is null ".equals(comparator)) {//不是is null，且不是SQL语句执行方式
                                dbFilter.add(filterMap.get("value"));//添加执行参数
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            System.out.println("getdbFilter 执行异常:".concat(LogUtils.getException(e)));
        }

        Map<String, Object> returnMap = new HashMap<>();
        returnMap.put("dbFilter", dbFilter);//执行条件参数
        returnMap.put("whereStr", whereStr);//执行条件where字符串
        return returnMap;
    }

    //上传条件参数的标准化
    private List<Map<String, Object>> standarInFilter(Map<String, Object> signParam, String event, List<Map<String, Object>> rowAuth) {
        Object filterObj = "2".equals(event) || signParam.containsKey("filter") ? signParam.get("filter") : signParam;//更新必须传filter，其它(查询和删除)可兼容上层
        List<Map<String, Object>> filterList = new ArrayList<>();
        if (Objects.nonNull(filterObj)) {//有上传条件
            if (filterObj instanceof Map) {// {"filter":{key,value}},{"filter":{left:""},{key,value}}
                filterList = changeSignFilter((Map<String, Object>) filterObj, "=", "and");//转换为标准的Filter列表
            } else {
                if (filterObj instanceof List<?> tempList) {//{"filter":[{key,value}]},{"filter":[{left,value}],[{key,value}]}
                    for (Object signFilter : tempList) {//循环所有上传条件参数
                        filterList.addAll(changeSignFilter((Map<String, Object>) signFilter, "=", "and"));//转换为标准的Filter列表
                    }
                }
            }
        }
        if (rowAuth.size() > 0) {//如果存在行权限
            if (filterList.size() > 0) { //存在上传的条件参数则组合 [{"left":,con}]
                filterList.get(0).put("left", "(".concat(filterList.get(0).get("left").toString()));//进行左右括号的包裹（and连接已经在行权限中处理过
                filterList.get(filterList.size() - 1).put("right", ")".concat(filterList.get(filterList.size() - 1).get("right").toString()));
                rowAuth.addAll(filterList);//行权限在前，上传条件在后
            }
            filterList = rowAuth;
        }
        return filterList;
    }

    //检测数据是否已经存在，使用同步锁(tableName,whereStr,filterList)
    private synchronized String isExist(String tableName, String whereStr, List<Object> dbFilter) {
        String event = "2";//默认为更新
        if (!AppConfig.event6Map.containsKey(tableName)) {//如果对应表名未建立缓存则初始化一个
            AppConfig.event6Map.put(tableName, new HashSet<>());
        }
        HashSet<Object> tableUniqe = AppConfig.event6Map.get(tableName);//获取已经缓存的数据
        String oldKey = whereStr.concat(MapTools.objToJSONStr(dbFilter));//条件字符串加条件执行参数
        if (!tableUniqe.contains(oldKey)) {//如果不存在则查数据库 // HashSet的包含效率高于list的包含
            try {           // isExistResult = null
                List<Map<String, Object>> isExistResult = theJdbcTemplate.queryForList("select 1 from ".concat(tableName).concat(" where ").concat(whereStr).concat(" limit 1"), dbFilter.toArray());
                event = (isExistResult.size() == 0) ? "1" : event;//数据库不存在则新增
                //如果是新增则修订为新增SQL
                if (tableUniqe.size() > 3000) {
                    tableUniqe.clear();
                }
                tableUniqe.add(oldKey);//无论是新增还是更新都添加到缓存
            } catch (Exception e) {//查询失败并不更新缓存
                //打印错误即可
                System.out.println(LogUtils.getException(e));
            }
        }
        return event;
    }

    //获取table执行的参数集：(tableName, event,authColumn,signParam)
    private Map<String, Object> getdbValueForTable(String tableName, String event, List<String> authColumn, Map<String, Object> signParam, boolean isActive) {
        List<Object> dbValue = new ArrayList<>();//初始化参数
        List<String> modifyValue = new ArrayList<>();
        List<String> allColumn = tableColumn.get(tableName);//初始化可用列默认等于所有列
        if (authColumn.size() > 0) {//存在列权限则处理列权限
            for (String sqlColumnName : authColumn) { //循环完整字段列表
                if (!authColumn.contains(sqlColumnName)) { //列权限不包含当前字段
                    allColumn.remove(sqlColumnName);//不存在于权限则删除
                }
            }
        }

        if ("1,2".contains(event)) {
//            List<String> priColumn = tablePrimaryKey.get(tableName);
//            priColumn.forEach(allColumn::remove);
            List<String> modyColumn = new ArrayList<>();
            Map<String, Object> currentParam = signParam.containsKey("Value") ? (Map<String, Object>) signParam.get("Value") : signParam;//如果存在value则取value否则取当前MAP
            for (String signKey : currentParam.keySet()) {//循环当前参数
                if (!allColumn.contains(signKey)) { //所有字段不包含当前字段则
                    if (isActive) {//如果是动态表则
                        theJdbcTemplate.execute("alter table ".concat(tableName).concat(" add ").concat(signKey).concat(" text "));//添加到数据库
                        allColumn.add(signKey);//添加到当前字段列表
                        tableColumn.get(tableName).add(signKey);//添加到缓存
                    } else {
                        continue;//不在当前字段列表也不是动态表的则跳过
                    }
                }
                Object currentValue = currentParam.get(signKey);
//                if ((Objects.nonNull(currentValue) && currentValue.toString().length() > 10 && currentValue.toString().charAt(10) == 'T')) {
//                    currentValue = currentValue.toString().replace("T", " ");
//                }
                modyColumn.add(signKey);
                dbValue.add((Objects.isNull(currentValue) && "createtime".equals(signKey)) ? new Date() : currentValue);//为空且字段名为createtime时做默认
                modifyValue.add("1".equals(event) ? "?" : signKey.concat("=?"));//新增替换为?，更新替换为字段名 = ?依据事件设置字段替换值
            }
            allColumn = modyColumn;
        }
        String sqlStr = AppConfig.eventSQL.get(event).replace("表名", tableName).replace("字段", String.join(",", allColumn)).replace("值", String.join(",", modifyValue));
        Map<String, Object> returnMap = new HashMap<>();
        returnMap.put("dbValue", dbValue);
        returnMap.put("sqlStr", sqlStr);
        returnMap.put("newSqlStr", sqlStr);
        returnMap.put("rowCountSQL", event.equals("0") ? "select count(1) from ".concat(tableName) : "");
        return returnMap;
    }

    //DB执行(signSql,execSQLMap.get(signSql),inputData)
    private Map<String, Object> execSign(String sqlStr, String signSql, Map<String, Object> sqlParmMap, Map<String, Object> inputData) {
        List<Object> dbValue = (List<Object>) sqlParmMap.get("dbParam");//提取执行参数
        if ("0".equals(sqlParmMap.get("event"))) {//如果是查询则
            if (Objects.nonNull(sqlParmMap.get("batchSQL"))) {
                signSql = sqlParmMap.get("batchSQL").toString();
                signSql = standarSQLStr(signSql);
                if (MapTools.isNotBlank(sqlStr)) {
                    String wherestr = signSql.replace(sqlStr, "");
                    if (!wherestr.equals(signSql)) {
                        if (wherestr.trim().startsWith("and")) {
                            wherestr = wherestr.trim().substring(0, 4).concat("(").concat(wherestr.trim().substring(4)).concat(")");
                        }
                        int groupByIndex = sqlStr.indexOf("group by");
                        if (groupByIndex > sqlStr.lastIndexOf("from")) {
                            signSql = sqlStr.substring(0, groupByIndex).concat(" ").concat(wherestr).concat(" ").concat(sqlStr.substring(groupByIndex));
                        } else {
                            signSql = sqlStr.concat(" ").concat(wherestr);
                        }
                    }
                }
            }

            if (signSql.lastIndexOf(")") >= signSql.lastIndexOf("limit")) {//如果是查询则进行翻页默认值处理,需要加入是否是数字的判断
                String page = (Objects.isNull(inputData) || Objects.isNull(inputData.get("page"))) ? "1" : inputData.get("page").toString();
                String pageSize = (Objects.isNull(inputData) || Objects.isNull(inputData.get("pageSize"))) ? "500" : inputData.get("pageSize").toString();
                int thePage = (!MapTools.isNumber(page)) ? 1 : Integer.parseInt(page);
                int thePageSize = (!MapTools.isNumber(pageSize)) ? 500 : Integer.parseInt(pageSize);
                signSql = signSql.concat((thePage == 1 && thePageSize == 1) ? " limit 1 " : (" limit ".concat(String.valueOf((thePage - 1) * thePageSize)).concat(" ,").concat(String.valueOf(thePageSize))));
            }

            List<Map<String, Object>> fullSelect = new ArrayList<>();
            for (Object signValue : dbValue) {
                try {//执行查询
                    Map<String, Object> returnData = processSuccess(theJdbcTemplate.queryForList(signSql, ((List) signValue).toArray()));
                    try {
                        if (Objects.nonNull(sqlParmMap.get("rowCountSql"))) {
                            returnData.put("rowcount", theJdbcTemplate.queryForObject(sqlParmMap.get("rowCountSql").toString(), Integer.class, ((List) signValue).toArray()));
                        }
                    } catch (Exception e) {
                        System.out.println("获取rowcount失败： ".concat(LogUtils.getException(e)));
                        returnData.put("rowcount", "0");
                    }

                    if (dbValue.size() == 1) {
                        return returnData;
                    }
                    if (returnData.get("code").equals("0")) {
                        fullSelect.addAll((Collection<? extends Map<String, Object>>) returnData.get("returnData"));
                    }
                } catch (Exception e) {
                    if (dbValue.size() == 1) {
                        return processFail("查询出现异常：".concat(LogUtils.getException(e)), signSql, dbValue);
                    }
                    System.out.println("批量查询异常，异常信息： " + LogUtils.getException(e));
                }
            }
            return processSuccess(fullSelect);
        }

        return JDBCBatch(signSql, dbValue, sqlParmMap.get("signParam"));
    }


    //JDBC更新
    public Map<String, Object> JDBCBatch(String sqlStr, List<Object> dbValue, Object signParam) {
        if ((sqlStr.startsWith("update") || sqlStr.contains("?")) && (dbValue.size() == 0 || (dbValue.get(0) instanceof List<?> list && list.isEmpty()))) {  //要求参数但是参数为空则返回
            return processSuccess(null);
        }
        Connection connection = null;
        PreparedStatement sqlPS = null;
        try {
            connection = theDataSource.getConnection();//获取连接
            sqlPS = connection.prepareStatement(sqlStr);//预编译SQL
            connection.setAutoCommit(false);    //手动事务
            for (int i = 0; i < dbValue.size(); i++) {

                Object[] signdbValue = ((List<Object>) dbValue.get(i)).toArray();
                for (int index = 0; index < signdbValue.length; index++) {
                    sqlPS.setObject(index + 1, signdbValue[index]);
                }
                sqlPS.addBatch();
                if ((i + 1) % 10 == 0) {
                    sqlPS.executeBatch();
                    connection.commit(); // 统一提交
                    sqlPS.clearBatch();
                }
            }
            sqlPS.executeBatch();
            connection.commit(); // 统一提交
            sqlPS.clearBatch();
            return processSuccess(dbValue);
        } catch (Exception e) {
            try {
                if (connection != null) {
                    connection.rollback();
                }
            } catch (SQLException ex) {
                System.out.println("jdbc 批量新增 : 事务回滚异常信息：".concat(LogUtils.getException(e)));
            }
            return processFail("执行批量更新SQL异常 ".concat(LogUtils.getException(e)).concat("事务回滚"), sqlStr, signParam);
        } finally {
            if (sqlPS != null) {
                try {
                    sqlPS.close();
                } catch (SQLException e) {
                    System.out.println("Statement 关闭连接失败");
                }
            }

            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException e) {
                    System.out.println("Statement 关闭连接失败");
                }
            }


        }
    }

    //对象关闭时需要关闭数据库连接池
    public void close() {
        try {
            errorMessage = "对象正在关闭";
            while (isActive) {
                Thread.sleep(1);
            }
            if (Objects.nonNull(theDataSource) && theDataSource.isRunning()) {//关闭连接池
                theDataSource.close();
            }
            theJdbcTemplate = null;//无条件情况即可
        } catch (Exception e) {
            System.out.println("mydbhepler 关闭异常： ".concat(LogUtils.getException(e)));
        }

    }


    //统一成功信息处理
    public Map<String, Object> processSuccess(Object returnData) {
        if (returnData instanceof Map) {
            Map<String, Object> resultData1 = new HashMap<>();
            try {
                resultData1 = (Map<String, Object>) returnData;
            } catch (Exception e) {
                resultData1.put("returnData", returnData);
            }
            resultData1.put("code", "0");
            return resultData1;
        }
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "0");
        returnMap.put("returnData", returnData);
        return returnMap;
    }


    //统一错误信息处理
    public Map<String, Object> processFail(String errorMessage, String sql, Object params) {
        Map<String, Object> returnMap = new HashMap<>();//用于当前方法统一返回参数，不存在深拷贝问题
        returnMap.put("code", "-1");
        returnMap.put("message", errorMessage);
        returnMap.put("sql", sql);
        returnMap.put("params", params);
        return returnMap;
    }

    /*执行创建表*/
    public Map<String, Object> execute(String sql) {
        try {
            theJdbcTemplate.execute(sql);
            return processSuccess(null);
        } catch (Exception e) {
            return processFail("执行创建表错误： ".concat(LogUtils.getException(e)), sql, null);
        }
    }

    //查询
    public Map<String, Object> queryByParamsReturnList(String sql, Object... params) {
        try {
            return processSuccess(theJdbcTemplate.queryForList(sql, params));
        } catch (Exception e) {
            return processFail("根据条件查询异常".concat(LogUtils.getException(e)), sql, params);
        }
    }

    //更新
    public Map<String, Object> updateByCondition(String sql, List<Object[]> batchPm, Object... params) {
        boolean bathpmNull = Objects.isNull(batchPm) || batchPm.size() == 0 || Objects.isNull(batchPm.get(0));
        try {
            if (sql.contains("?") && (bathpmNull && params.length != StringUtils.countOccurrencesOf(sql, "?"))) {
                return processSuccess(null);
            }
            Object tempObject = bathpmNull ? theJdbcTemplate.update(sql, params) : theJdbcTemplate.batchUpdate(sql, batchPm);
            return processSuccess(tempObject);
        } catch (Exception e) {
            return processFail("根据条件更新异常,SQL: ".concat(LogUtils.getException(e)), sql, bathpmNull ? batchPm : params);
        }
    }

    //查询数据库条数
    public Map<String, String> queryForObject(String sql, Object... params) {
        HashMap<String, String> returnMap = new HashMap<>();
        returnMap.put("code", "0");
        returnMap.put("sql", sql);
        returnMap.put("params", Arrays.toString(params));
        try {
            returnMap.put("returnData", String.valueOf(theJdbcTemplate.queryForObject(sql, Integer.class, params)));
        } catch (Exception e) {
            returnMap.put("message", "查询条数 执行异常： ".concat(LogUtils.getException(e)));
        }
        return returnMap;
    }

    /***
     * jdbcTemplate 插入数据后返回，主键的值
     * @param sql  sql语句
     * @param params sql参数
     * @return 插入的主键ID
     */
    public Map<String, String> insertReturnKeyValues(String sql, Object... params) {
        HashMap<String, String> returnMap = new HashMap<>();
        returnMap.put("code", "0");
        returnMap.put("sql", sql);
        returnMap.put("params", Arrays.toString(params));

        try {
            KeyHolder holder = new GeneratedKeyHolder();
            theJdbcTemplate.update(connection -> {
                PreparedStatement ps = connection.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
                for (Object param : params) {
                    ps.setObject(1, param);
                }
                return ps;
            }, holder);
            returnMap.put("returnData", String.valueOf(Objects.requireNonNull(holder.getKey()).intValue()));
        } catch (Exception e) {
            returnMap.put("message", "insertReturnKeyValues错误：".concat(LogUtils.getException(e)));
        }
        return returnMap;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    public HikariDataSource getTheDataSource() {
        return theDataSource;
    }

    public JdbcTemplate getTheJdbcTemplate() {
        return theJdbcTemplate;
    }

    public List<Map<String, Object>> changeSignFilter(Map<String, Object> filterMap, String compare, String connect) {
        List<Map<String, Object>> objects = new ArrayList<>();
        if (filterMap.containsKey("left")) {
            objects.add(filterMap);
            return objects;
        }
        for (String filterKey : filterMap.keySet()) {
            Map<String, Object> returnMap = new HashMap<>();
            Object keyValues = filterMap.get(filterKey);
            returnMap.put("left", "");
            returnMap.put("column", filterKey);
            returnMap.put("comparator", (Objects.nonNull(compare) && compare.startsWith("!")) ? "!=" : " = ");
            if (Objects.isNull(keyValues)) {
                returnMap.put("comparator", " is null ");
            }
            returnMap.put("value", keyValues);
            returnMap.put("right", "");
            returnMap.put("connector", Objects.nonNull(connect) ? " ".concat(connect).concat(" ") : " and ");
            objects.add(returnMap);
        }
        return objects;
    }
}
