package org.bfkj.utils;

import org.bfkj.config.AppConfig;
import org.bfkj.config.ObjectMap;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;

import java.util.*;
import java.util.concurrent.ScheduledFuture;

/**
 * 定时任务工具类
 *
 * @author zhongxiaojian
 * @date 2020/4/17
 **/
public class ScheduleUtil {
    private static ThreadPoolTaskScheduler threadPoolTaskScheduler = new ThreadPoolTaskScheduler();

    private static Map<String, ScheduledFuture<?>> scheduledFutureMap = new HashMap<>();

    static {

        threadPoolTaskScheduler.setPoolSize(8);
        threadPoolTaskScheduler.initialize();
        System.out.println("定时任务线程池启动");
    }

    /**
     * 启动
     *
     * @param scheduleTask 定时任务
     */
    public static boolean start(ScheduleTask scheduleTask, int cycleFrequency, Object cronExpress, int taskType) {
        System.out.println("启动定时任务线程 taskId " + scheduleTask.getId());// 设置时间比当前时间晚了
        ScheduledFuture<?> scheduledFuture;
        if (taskType == 1) {
            scheduledFuture = threadPoolTaskScheduler.scheduleWithFixedDelay(scheduleTask, new Date(), cycleFrequency);
        } else {
            if (Objects.isNull(cronExpress)) {
                LogUtils.log("ScheduleUtil: 2", "0",  null,"定时任务表达式为空", scheduleTask.getId(), null, null,"定时任务表达式为空",null);//无条件记录数据接收日志
                return false;
            }
            scheduledFuture = threadPoolTaskScheduler.schedule(scheduleTask, new CronTrigger(cronExpress.toString()));
        }
        scheduledFutureMap.put(scheduleTask.getId(), scheduledFuture);
        return true;
    }

    /**
     * 取消
     *
     * @param scheduleTask 定时任务
     */
    public static boolean cancel(ScheduleTask scheduleTask) {
        System.out.println("关闭定时任务线程 taskId " + scheduleTask.getId());
        String serviceId = scheduleTask.getId();
        ScheduledFuture<?> scheduledFuture = scheduledFutureMap.get(serviceId);
        if (scheduledFuture != null && !scheduledFuture.isCancelled()) {
            scheduledFuture.cancel(false);
        }
        scheduledFutureMap.remove(serviceId);
        //根据服务ID获取所有算法，进行统一关闭
        ObjectMap.getordropInput(serviceId,false);
        return true;
    }
}

